/*
 
SensorControlModule.ino

2014/6/3    version 2.1
2013/9/10   version 2.0
2013/5/6    version 1.1
2012/10/26  version 1.0
 
copyright(c) 2014 All Blue System

----------------------------------------------------------------------
■始めに
----------------------------------------------------------------------

このスケッチは、シリアルポートからFirmata プロトコルでリクエストされた
コマンドによってデジタル I/O ポートの操作を行います。また、I2C マスター
としてI2C スレーブデバイスをコントロールすることもできます。

I2Cスレーブデバイスとしても動作して、I2Cバスに接続したマスターデバイス
からこのArduinoにスレーブコマンドを送信して、サーバーPC にFirmata
イベントメッセージを送信することができます。

標準で定義されたプロトコルの他に、Firmata の SYSEX 拡張フォーマットを
使用して独自にコマンドを定義しています。
詳細は、"サポートしている Firmata プロトコル" の項目をご覧下さい。

----------------------------------------------------------------------
■使用するにあたって
----------------------------------------------------------------------

個人利用または商用利用を問わずにこのファイルを自由に使用できます。
ファイルの内容を改変したり、一部分を取り出して利用することもできます。

オールブルーシステムは、このファイルを使用したことによる損害等につい
て一切保障できません。

--------------------------------------------------------------------
■このファイルの動作環境
--------------------------------------------------------------------

このファイルは Arduino Duemilanove 328 互換機で動作確認を行っています。
ATmega328P以外の CPU を使用している Arduino や FT232RL以外の
USB-シリアル変換デバイスを使用した Arduino 互換ボードでの動作確認は
していません。

このファイルは ver1.0.1-1.0.5,1.6.0 for Windows で動作確認を行って
います。これより古いバージョンの Arduino IDE では動作しない場合があ
ります。また Windows 以外の OS では動作確認していません。

--------------------------------------------------------------------
■このファイルを使用する前に ArduinoIDE のファイルを修正していください
--------------------------------------------------------------------
Arduino IDE ver1.0.1-1.0.5,1.6.0 for Windows では下記のパッチを当てる必要があ
ります(ver1.0.1 の場合を例にして説明します)


(1) Arduino がスレーブデバイスとして他の I2C マスターから REPEATED 
　　START モードでアクセスする部分のコードを下記のように変更してください。


	下記ファイル
    arduino-1.0.1-windows/arduino-1.0.1/libraries/Wire/utility/twi.c または
	arduino-1.6.0-windows/arduino-1.6.0/hardware/arduino/avr/libraries/Wire/utility/twi.c
	
　　ファイル中の下記の部分を１行コメントアウトする

    ...
    ...
    case TW_SR_STOP: // stop or repeated start condition received
      // put a null char after data if there's room
      if(twi_rxBufferIndex < TWI_BUFFER_LENGTH){
        twi_rxBuffer[twi_rxBufferIndex] = '\0';
      }
      // sends ack and stops interface for clock stretching
      // ***********************************************
　　  // ******* この下の1行をコメントアウトする *******
      // ***********************************************
　　　//twi_stop();
      // callback to user defined callback
      twi_onSlaveReceive(twi_rxBuffer, twi_rxBufferIndex);
      // since we submit rx buffer to "wire" library, we can reset it
      twi_rxBufferIndex = 0;
      // ack future responses and leave slave receiver state
      twi_releaseBus();
      break;
    ...
    ...

    この変更を行わなくても、I2C スレーブコマンドの write 部分だけを最初に
    MASTER TRANSMIT で書き込んだ後、MASTER RECEIVE で読み込むように２回に分けて
　　アクセスすることで、REPEATED START モードを使用しない方法でアクセスすること
　　ができます。


(2) Firmata 標準ライブラリの最大受信データ数を増やす。

	下記ファイル
    arduino-1.0.1-windows/arduino-1.0.1/libraries/Firmata/Firmata.h または
	arduino-1.6.0-windows/arduino-1.6.0/libraries/Firmata/src/Firmata.h

    の定義部分を下記の様に変更してください。

    //#define MAX_DATA_BYTES 32 // max number of data bytes in non-Sysex....
    #define MAX_DATA_BYTES 70   // max number of data bytes in non-Sysex....


--------------------------------------------------------------------
■Arduino ピンコンフィギュレーション
--------------------------------------------------------------------

このプログラムで割り当てた機能と使用する Arduino ピン番号について

* H/W SERIAL = D0,D1

* DIGITAL PORT(INPUT/OUTPUT) = D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13

  INPUT/OUTPUT のピンの割り当て変更は、SensorControlModule.ino 
  ファイル中の setup() 関数内のコメントを参照してください

  D10,D11,D12,D13 は SPI 機能を使用するときには DIGITAL PORTとしては
  使用できません。デフォルトでは SPI 機能が有効になっていますので、
  D10-D13 をデジタルポートとして使用する場合にはこのファイル中の
  以下の部分

  #define USE_SPI_CONFIG    1

  を

  //#define USE_SPI_CONFIG    1
  
  の様にコメントアウトしてください。
      
* ANALOG PORT(INPUT) = A0,A1,A2,A3

* I2C
  I2C SDA = A4
  I2C SCL = A5

* SPI (上記 DIGITAL PORT の項も参照してください)
  SPI SS   = D10 (low active)
  SPI MOSI = D11
  SPI MISO = D12
  SPI SCK  = D13

--------------------------------------------------------------------
■サポートしている Firmata プロトコル（コマンド）
--------------------------------------------------------------------
 
[DIGITAL_MESSAGE]
   
   標準プロトコルで定義されているプロトコル。
   PC から Arduinoボード側へのI/O ポート値設定のみに使用しています。

   0  DIGITAL_MESSAGE      (0x90) + port#
   1  port value bits 0-6  (LSB)
   2  port value bits 7-13 (MSB)
  
[DIGITAL_PORTS_QUERY]
 
   全てのデジタルポートの現在の値を取得するためのリクエストコマンド
   
   0  START_SYSEX          (0xF0)
   1  DIGITAL_PORTS_QUERY  (0x61)
   2  END_SYSEX            (0xF7)
  
[DIGITAL_PORTS_REPLY]

   DIGITAL_PORTS_QUERY コマンドのリプライメッセージパケット。

   0  START_SYSEX          (0xF0)
   1  DIGITAL_PORTS_REPLY  (0x62)
   2  port#0 bits 0-6      (LSB)
   3  port#0 bits 7-13     (MSB)
   ..
   4  port#n bits 0-6      (LSB)
   5  port#n bits 7-13     (MSB)
   6  END_SYSEX            (0xF7)

[DIGITAL_WRITE_PIN]

  デジタルポートの指定したピンの値をセットする
  
   0  START_SYSEX          (0xF0)
   1  DIGITAL_WRITE_PIN    (0x63)
   3  pin number           (0x02-0x0D)
   4  pin value            (0/1)
   5  END_SYSEX            (0xF7)

[DIGITAL_CHANGE_DETECT]

   I/O 値が更新された場合に、PC 側に最新のI/O ポート値を送信する

   0  START_SYSEX          (0xF0)
   1  DIGITAL_CHANGE_DETECT(0x6B)
   2  port#0 diff bits 0-6 (LSB)
   3  port#0 diff bits 7-13(MSB)
   4  port#0 bits 0-6      (LSB)
   5  port#0 bits 7-13     (MSB)
   ..
   6  port#n diff bits 0-6 (LSB)
   7  port#n diff bits 7-13(MSB)
   8  port#n bits 0-6      (LSB)
   9  port#n bits 7-13     (MSB)
  10  END_SYSEX            (0xF7)

[DIGITAL_CHANGE_MASK]

   DIGITAL_CHANGE_DETECTで検出対象とするポートのビットマスク値を指定する
   ビットが 1 の場合に DIGITAL_CHANGE_DETECT 検出対象となる
   初期値は、全ポートのビットマスクがリセットされて非検出対象になっています。
   setup() 関数内のコメントを参照してください

   0  START_SYSEX          (0xF0)
   1  DIGITAL_CHANGE_MASK  (0x6C)
   2  port number	  	   (0x00-0x7F)
   3  port mask bits 0-6   (LSB)
   4  port mask bits 7-13  (MSB)
   5  END_SYSEX            (0xF7)

[DIGITAL_CONFIG_QUERY]
 
   デジタルポートの現在の入出力モードを取得するリクエストコマンド
   
   0  START_SYSEX          (0xF0)
   1  DIGITAL_PORTS_QUERY  (0x55)
   2  END_SYSEX            (0xF7)
  
[DIGITAL_CONFIG_REPLY]

   DIGITAL_CONFIG_QUERY コマンドのリプライメッセージパケット。

   0  START_SYSEX             (0xF0)
   1  DIGITAL_CONFIG_REPLY    (0x56)
   2  spi use                 (0x00 or 0x01)  SPI を使用している場合には 0x01 が入る
   3  port_config#0 bits 0-6  (LSB)
   4  port_config#0 bits 7-13 (MSB)
   ..
   5  port_config#n bits 0-6  (LSB)
   6  port_config#n bits 7-13 (MSB)
   7  END_SYSEX               (0xF7)

[I2C_WRITE_AND_READ_QUERY]
  
   I2C スレーブデバイスにデータを書き込んだ後、同一スレーブデバイスから
   データを読み込む
  
   0  START_SYSEX              (0xF0)
   1  I2C_WRITE_AND_READ_QUERY (0x65)
   2  7bit slave address       (0x01-0x7F)
   3  read data count          (0-32)
   4  write#1 bits 0-6         (LSB)
   5  write#1 bits 7-13        (MSB)
   ..
   6  write#n bits 0-6         (LSB)
   7  write#n bits 7-13        (MSB) 
   8  END_SYSEX                (0xF7)
  
   * 書き込みデータ write#n の最大エントリ数は 31
  
[I2C_READ_REPLY]

   I2C_READ_QUERY または I2C_WRITE_AND_READ_QUERYコマンドの
   リプライメッセージパケット。スレーブデバイスから読み込んだ
   データが格納される

   0  START_SYSEX          (0xF0)
   1  I2C_READ_REPLY       (0x66)
   3  read#1 bits 0-6      (LSB)
   4  read#1 bits 7-13     (MSB)
   ..
   5  read#n bits 0-6      (LSB)
   6  read#n bits 7-13     (MSB)
   7  END_SYSEX            (0xF7)
   
   * 読み込みデータ read#n の最大エントリ数は 32

[I2C_WRITE_COMMAND]
  
   I2C スレーブデバイスにデータを書き込む
  
   0  START_SYSEX              (0xF0)
   1  I2C_WRITE_COMMAND        (0x67)
   2  7bit slave address       (0x01-0x7F)
   3  write#1 bits 0-6         (LSB)
   4  write#1 bits 7-13        (MSB)
   ..
   5  write#n bits 0-6         (LSB)
   6  write#n bits 7-13        (MSB) 
   7  END_SYSEX                (0xF7)
  
   * 書き込みデータ write#n の最大エントリ数は 31

[I2C_READ_QUERY]
  
   I2C スレーブデバイスからデータを読み込む
  
   0  START_SYSEX              (0xF0)
   1  I2C_READ_QUERY           (0x68)
   2  7bit slave address       (0x01-0x7F)
   3  read data count          (1-32)
   4  END_SYSEX                (0xF7)
  
[I2C_EVENT_DATA]
   
   I2C スレーブデバイスからイベント送信コマンド(0x11) を受信したときに
   サーバーにそのイベントデータを送信するためのパケットデータ。

   0  START_SYSEX              (0xF0)
   1  I2C_EVENT_DATA           (0x69)
   3  event#1 bits 0-6         (LSB)
   4  event#1 bits 7-13        (MSB)
   ..
   5  event#n bits 0-6         (LSB)
   6  event#n bits 7-13        (MSB) 
   7  END_SYSEX                (0xF7)
   
   * 書き込みデータ write#n の最大エントリ数は 31
   
[I2C_EVENT_OVERFLOW]
   
   I2C スレーブデバイスからイベント送信コマンド(0x11) を受信したときに
   イベントバッファがオーバーフローしたことをサーバーに知らせるパケットデータ。

   0  START_SYSEX              (0xF0)
   1  I2C_EVENT_OVERFLOW       (0x6A)
   2  END_SYSEX                (0xF7)

[ANALOG_READ_QUERY]
 
   アナログポートの現在の値を取得するためのリクエストコマンド
   
   0  START_SYSEX          (0xF0)
   1  ANALOG_READ_QUERY    (0x51)
   2  END_SYSEX            (0xF7)
  
[ANALOG_READ_REPLY]

   ANALOG_READ_QUERY コマンドのリプライメッセージパケット。

   0  START_SYSEX          (0xF0)
   1  ANALOG_READ_REPLY    (0x52)
   3  ad#0 bits 0-6        (LSB)
   4  ad#0 bits 7-13       (MSB)
   ..
   5  ad#n bits 0-6        (LSB)
   6  ad#n bits 7-13       (MSB)
   7  END_SYSEX            (0xF7)

   * 取得するアナログポート ad#n は 0 から 3 までの4つです

[ANALOG_MARGIN]

   アナログ入力の変化検出のマージン値を設定する
   0 の場合(初期値)は変化を検出しない

   0  START_SYSEX           (0xF0)
   1  ANALOG_MARGIN         (0x53)
   2  analong pin number    (0-3)
   3  ad_margin#n bits 0-6  (LSB)
   4  ad_margin#n bits 7-13 (MSB)
   5  END_SYSEX             (0xF7)

[ANALOG_UPDATE]

   ANALOG_MARGIN で設定した値よりもアナログ入力値が変化したばあいに
   サーバーに送信される現在のアナログデータが格納されたイベントデータ

   0  START_SYSEX          (0xF0)
   1  ANALOG_UPDATE    (0x54)
   2  ad_update_bits       (0x00-0x0F) ad#0(bit0) .. ad#3(bit3) に対応する
   3  ad#0 bits 0-6        (LSB)
   4  ad#0 bits 7-13       (MSB)
   ..
   5  ad#n bits 0-6        (LSB)
   6  ad#n bits 7-13       (MSB)
   7  END_SYSEX            (0xF7)

   * ad_update_bits には、変化したアナログポートの番号に一致するビット値が 1 に設定されます
   例えば、ad_update_bits が 0x03 の場合には ad#0, ad#1 の両方が、前回 ad#0, ad#1 のそれぞれ
   に対するA/D 変化イベントを検出してから ANALOG_MARGIN(0),ANALOG_MARGIN(1)で設定した値以上
   に変化したことを示します

   * 送信されるアナログポート ad#n は 0 から 3 までの4つです

[SPI_READ_QUERY]
  
   SPI スレーブデバイスからデータを読み込む
  
   0  START_SYSEX              (0xF0)
   1  SPI_READ_QUERY           (0x57)
   2  read data count          (1-32)
   3  END_SYSEX                (0xF7)
  
[SPI_WRITE_READ_QUERY]
  
   I2C スレーブデバイスにデータを書き込んだ後、同一スレーブデバイスから
   データを読み込む
  
   0  START_SYSEX              (0xF0)
   1  SPI_WRITE_READ_QUERY     (0x58)
   2  write#1 bits 0-6         (LSB)
   3  write#1 bits 7-13        (MSB)
   ..
   4  write#n bits 0-6         (LSB)
   5  write#n bits 7-13        (MSB) 
   6  END_SYSEX                (0xF7)
  
   * 書き込みデータ write#n の最大エントリ数は 31

[SPI_READ_REPLY]

   SPI_READ_QUERY または SPI_WRITE_READ_QUERY コマンドの
   リプライメッセージパケット。スレーブデバイスから読み込んだ
   データが格納される

   0  START_SYSEX          (0xF0)
   1  SPI_READ_REPLY       (0x59)
   2  read#1 bits 0-6      (LSB)
   3  read#1 bits 7-13     (MSB)
   ..
   4  read#n bits 0-6      (LSB)
   5  read#n bits 7-13     (MSB)
   6  END_SYSEX            (0xF7)
   
   * 読み込みデータ read#n の最大エントリ数は 32

[SPI_WRITE]
  
   I2C スレーブデバイスにデータを書き込む
  
   0  START_SYSEX              (0xF0)
   1  SPI_WRITE                (0x5A)
   2  write#1 bits 0-6         (LSB)
   3  write#1 bits 7-13        (MSB)
   ..
   4  write#n bits 0-6         (LSB)
   5  write#n bits 7-13        (MSB) 
   6  END_SYSEX                (0xF7)
  
   * 書き込みデータ write#n の最大エントリ数は 31

[DIGITAL_BLINK_MASK]

  指定したビットに対応するデジタルポートピンをブリンク表示にする
  700ms 間隔で HIGH, LOW の繰り返し出力を行う。

   0  START_SYSEX             (0xF0)
   1  DIGITAL_BLINK_MASK      (0x5B)
   2  port number	      (0x00-0x7F)
   3  port bits 0-6           (LSB)
   4  port bits 7-13          (MSB)
   5  END_SYSEX               (0xF7)

--------------------------------------------------------------------
■I2C Slave モードコマンド一覧
--------------------------------------------------------------------

(1) アプリケーションモード(app_mode) 取得
---------------+--------------------------
write[0]       | <i2c command> : 0x01
---------------+--------------------------
read[0]        | <app_mode> see ARDUINO_PROGRAM_APP_MODE
---------------+--------------------------

(2) イベント送信

最大 31バイト分のイベントデータを送信指示できる
"I2C_SLAVE_EVENT" Firmata パケットとしてDeviceServer に送信される

---------------+-----------------------------------------
write[0]       | <i2c command> : 0x11
---------------+-----------------------------------------
write[1]       | <event data#0>
---------------+-----------------------------------------
write[2]       | <event data#1>
---------------+-----------------------------------------
              ..
---------------+-----------------------------------------
write[31]      | <event data#30>
---------------+-----------------------------------------

*/


#include <Firmata.h>
#include <Wire.h>
#include <SPI.h>
#include <util/atomic.h>

//  デジタルポート数の定義
//  ATmega328 を使用した Arduino ボードの場合は 2 になる
//  通常はこのまま変更する必要はありません。
#define TOTAL_DIGITAL_PORTS             ((TOTAL_PINS - TOTAL_ANALOG_PINS + 7) / 8)

//  使用するアナログポート数の定義
//  ganalogRead(analog_pin) の analog_pin で指定する範囲: 0.. (USE_ANALOG_PIN_COUNT - 1)
//  analog_pin :4,5 は I2C バスで使用するためにアナログでは使用しない
//  通常はこのまま変更する必要はありません。
#define USE_ANALOG_PIN_COUNT	4

// SPI を使用するかどうかの設定
// SPI を使用するときにはデジタルポートD10-D13 が SPI インターフェイス用に設定される
// SS   = D10 (OUTPUT)
// MOSI = D11 (OUTPUT)
// MISO = D12 (INPUT)
// SCK  = D13 (OUTPUT)
// 
// SPI 機能を使用しない場合で D10,D11,D12,D13 を デジタルポートとして使用したい場合には
// この値を 0 または定義文をコメントアウトしてください。
#define USE_SPI_CONFIG    1

// ログメッセージ機能を使用するかどうかを指定する
// このスケッチプログラム中からログメッセージを Firmata プロトコル経由で出力するときに
// ワークエリアとして svr_log_buff[] バッファ 80(bytes)分のメモリを確保します。
#define LOG_MESSAGE	1


// SYSEX コマンドの定義
#define DIGITAL_PORTS_QUERY      0x61
#define DIGITAL_PORTS_REPLY      0x62
#define DIGITAL_WRITE_PIN        0x63
//#define DIGITAL_PORTS_MESSAGE    0x64

#define I2C_WRITE_AND_READ_QUERY    0x65
#define I2C_READ_REPLY         0x66
#define I2C_WRITE_COMMAND      0x67
#define I2C_READ_QUERY         0x68
#define I2C_EVENT_DATA         0x69
#define I2C_EVENT_OVERFLOW     0x6A

#define DIGITAL_CHANGE_DETECT  0x6B
#define DIGITAL_CHANGE_MASK    0x6C

#define ANALOG_READ_QUERY      0x51
#define ANALOG_READ_REPLY      0x52
#define ANALOG_MARGIN          0x53
#define ANALOG_UPDATE          0x54

#define DIGITAL_CONFIG_QUERY   0x55  // port 毎に pin モードが INPUT/OUTPUT のどちらに設定されているかを調べる
#define DIGITAL_CONFIG_REPLY   0x56

#define SPI_READ_QUERY         0x57  // slave デバイスには読み込みバイト数で指定したのと同じ長さの 0x00 が送信される
#define SPI_WRITE_READ_QUERY   0x58
#define SPI_READ_REPLY         0x59
#define SPI_WRITE              0x5A  // リプライデータの応答は無しで書き込み動作のみ

#define DIGITAL_BLINK_MASK     0x5B

//ログメッセージ表示用バッファ
#ifdef	LOG_MESSAGE
char svr_log_buff[80];
#endif

// Arduino 自身の I2C スレーブアドレス
#define ARDUINO_I2C_SLAVE_ADDR  0x33

// I2C スレーブコマンド 0x01(アプリケーションモード問い合わせ)で応答する
// アプリケーションモード を定義する
#define ARDUINO_PROGRAM_APP_MODE  41

// I2C データ書き込みリクエストと受信データリプライ用のデータバッファ
#define I2C_DATA_BUFF_SIZE 64
volatile byte i2c_req_size;
byte i2c_req_buff[I2C_DATA_BUFF_SIZE];
volatile byte i2c_rpl_size;
byte i2c_rpl_buff[I2C_DATA_BUFF_SIZE];

// I2C スレーブコマンド処理用データバッファ
uint8_t i2c_prev_rxBuffer[I2C_DATA_BUFF_SIZE];
volatile uint8_t i2c_prev_rxBufferIndex;

// I2C スレーブコマンド(0x11) イベント送信用バッファ 
#define I2C_EVENT_BUFF_SIZE 5
#define I2C_EVENT_LENGTH 32
uint8_t i2c_event_data_buff[I2C_EVENT_BUFF_SIZE][I2C_EVENT_LENGTH];
uint8_t i2c_event_data_size[I2C_EVENT_BUFF_SIZE]; 
volatile uint8_t event_overflow_occur;
volatile uint8_t event_buff_write_ptr;
volatile uint8_t event_buff_read_ptr;

// デジタルポートの値が変更されかどうかを判断するための値保存用
volatile byte stable_prev_dio[TOTAL_DIGITAL_PORTS];
volatile byte stable_current_dio[TOTAL_DIGITAL_PORTS];
volatile byte prev_dio[TOTAL_DIGITAL_PORTS];
volatile byte current_dio[TOTAL_DIGITAL_PORTS];
volatile byte f_change_detect[TOTAL_DIGITAL_PORTS];
volatile uint8_t first_dio_sample;

// 現在の デジタルポートの入出力モード設定値保持用
// 実際のモードは setup() 関数内で pinMode() 関数を使用して
// この設定値の内容を反映させている
volatile byte f_dio_config[TOTAL_DIGITAL_PORTS];

// アナログポートの値が変更されかどうかを判断するための値保存用
volatile int  stable_adc[USE_ANALOG_PIN_COUNT];
volatile int  f_ad_margin[USE_ANALOG_PIN_COUNT];
volatile uint8_t first_ad_sample;

// SPI データ書き込みリクエストと受信データリプライ用のデータバッファ
#define SPI_BUFFER_LENGTH 32
volatile byte spi_req_size;
byte spi_req_buff[SPI_BUFFER_LENGTH];
volatile byte spi_rpl_size;
byte  spi_rpl_buff[SPI_BUFFER_LENGTH];

// BLINK モード設定用ビット値
volatile byte f_blink_pins[TOTAL_DIGITAL_PORTS];

// SPI スレーブセレクトを High にする
void ss_pin_high(void)
{
    digitalWrite(10,HIGH);  // SPI slave select to HIGH
}

// SPI スレーブセレクトを Low にする
void ss_pin_low(void)
{
    digitalWrite(10,LOW);   // SPI slave select to LOW
}

// SPI スレーブデバイス間でデータ送受信を行う
uint8_t spi_write_and_receive(uint8_t* tx_data, uint8_t tx_length,uint8_t* rx_data)
{
  uint8_t i;

  // SS enable
  ss_pin_low();
  delayMicroseconds(1);

  // ensure data will fit into buffer
  if(SPI_BUFFER_LENGTH < tx_length){
      return 0;	// error
  }
  // send and receive data
  for(i = 0; i < tx_length;i++){
      rx_data[i] =  SPI.transfer(tx_data[i]);
  }

  // SS disable
  delayMicroseconds(1);
  ss_pin_high();

  return tx_length;
}

// イベント送信用データをバッファに格納する
// return:
//        0: 正常に格納した
//	  1: バッファが一杯で格納に失敗した。
uint8_t i2c_event_buff_add(uint8_t* in_data,uint8_t in_data_len)
{
    uint8_t	found = 0;
    for(int i = 0;i < I2C_EVENT_BUFF_SIZE;i++){
        if(i2c_event_data_size[event_buff_write_ptr] == 0){
	    memcpy(&i2c_event_data_buff[event_buff_write_ptr][0],in_data,in_data_len);
	    i2c_event_data_size[event_buff_write_ptr] = in_data_len; 
	    found = 1;
	    break;
	}
	if(event_buff_write_ptr == I2C_EVENT_BUFF_SIZE - 1) {
  	    event_buff_write_ptr = 0;
	} else {
	    event_buff_write_ptr++;
	}
    }
    event_overflow_occur = !found;
    return found;
}

// イベント送信用データをバッファから取り出す。
// return:
//        0: 有効なイベントデータが見つからなかった。
//	  1: 有効なイベントデータが見つかったまたは、オーバーフロー状態を確認した
uint8_t i2c_event_buff_extract(uint8_t* out_data,uint8_t* out_data_len,uint8_t* overflow_flag)
{
    uint8_t	found = 0;
    if(event_overflow_occur){
	*overflow_flag = 1;
	event_overflow_occur = 0;
	found = 1;
    } else {
	*overflow_flag = 0;
        for(int i = 0;i < I2C_EVENT_BUFF_SIZE;i++){
	    if(i2c_event_data_size[event_buff_read_ptr]){
	        memcpy(out_data,&i2c_event_data_buff[event_buff_read_ptr][0],i2c_event_data_size[event_buff_read_ptr]);
		*out_data_len = i2c_event_data_size[event_buff_read_ptr]; 
		i2c_event_data_size[event_buff_read_ptr] = 0;
		found = 1;
		break;
	    }
	    if(event_buff_read_ptr == I2C_EVENT_BUFF_SIZE - 1) {
		event_buff_read_ptr = 0;
	    } else {
		event_buff_read_ptr++;
	    }
        }
    }
    return found;
}

// イベント送信用バッファ初期化
void i2c_event_buff_init(void)
{
    for(int i = 0;i < I2C_EVENT_BUFF_SIZE;i++){
        i2c_event_data_buff[i][0] = 0x0;
	i2c_event_data_size[i] = 0;
    }
    event_buff_write_ptr = 0;
    event_buff_read_ptr = 0;
    event_overflow_occur = 0;
}

// 定期的にコールされて、I2C スレーブコマンドによってイベント送信用のデータがバッファに
// 格納されていた場合には、FIRMATA メッセージを送信する
void i2cEvnetTask(void){
    char	prt_item[3];
    uint8_t	stat;
    uint8_t	event_data[I2C_EVENT_LENGTH];
    uint8_t	event_data_len;
    uint8_t	event_overflow = 0;

    ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
    {
        stat = i2c_event_buff_extract(event_data,&event_data_len,&event_overflow);
    }	

    if(stat) {
        if(event_overflow){
            Serial.write(START_SYSEX);
            Serial.write(I2C_EVENT_OVERFLOW);
            Serial.write(END_SYSEX);
	} else {
            Serial.write(START_SYSEX);
            Serial.write(I2C_EVENT_DATA);
	    for(int i = 0;i < event_data_len;i++){
                Serial.write(event_data[i] & 0x7F);        // LSB
                Serial.write((event_data[i] >> 7) & 0x7F); // MSB
	    }
            Serial.write(END_SYSEX);
	}
    }
}

// Wire ライブラリで使用しているイベントハンドラ
void i2cRequestEvent()
{
    if ((i2c_prev_rxBufferIndex == 0) || (i2c_prev_rxBuffer[0] == 0)) return;
    switch(i2c_prev_rxBuffer[0]){
        case 0x01:  // get app_mode
                    Wire.write((uint8_t)ARDUINO_PROGRAM_APP_MODE);
  		    break;
	default:
	    	    break;
    }
}

// Wire ライブラリで使用しているイベントハンドラ
void i2cReceiveEvent(int howMany)
{
    i2c_prev_rxBufferIndex = howMany;
    if (i2c_prev_rxBufferIndex < 1) return;
    for(int i = 0;i < i2c_prev_rxBufferIndex;i++){
        i2c_prev_rxBuffer[i] = Wire.read();
    }
    switch(i2c_prev_rxBuffer[0]){
	case 0x11:  // send slave event to the DeviceServer
		    i2c_event_buff_add(&i2c_prev_rxBuffer[1],i2c_prev_rxBufferIndex - 1);
      		    break;
    	default:
  		    break;
    }
}

// i2c_rpl_buff,i2c_rpl_size にセットされたバイト列を
// SYSEX コマンド I2C_READ_REPLY で送信する
void replayI2CReadData(void){
    Serial.write(START_SYSEX);
    Serial.write(I2C_READ_REPLY);
    for (byte i=0; i<i2c_rpl_size; i++) {
        Serial.write(i2c_rpl_buff[i] & 0x7F);        // LSB
        Serial.write((i2c_rpl_buff[i] >> 7) & 0x7F); // MSB
    }
    Serial.write(END_SYSEX);
}

// spi_rpl_buff,spi_rpl_size にセットされたバイト列を
// SYSEX コマンド SPI_READ_REPLY で送信する
void replaySPIReadData(void){
    Serial.write(START_SYSEX);
    Serial.write(SPI_READ_REPLY);
    for (byte i=0; i<spi_rpl_size; i++) {
        Serial.write(spi_rpl_buff[i] & 0x7F);        // LSB
        Serial.write((spi_rpl_buff[i] >> 7) & 0x7F); // MSB
    }
    Serial.write(END_SYSEX);
}

// SYSEX コマンドの処理
void sysexCallback(byte command, byte argc, byte *argv)
{
    byte pval;
    byte chg_val;
    byte i2c_slave_addr;
    byte read_cnt;
    int  adval;
    
    switch(command) {
      case I2C_WRITE_COMMAND:
            i2c_slave_addr = argv[0];
            // prepare write data
            i2c_req_size = (argc - 1) / 2;             
            for(byte i=0;i < i2c_req_size;i++){
                i2c_req_buff[i] = argv[1 + i * 2] + (argv[1 + i * 2 + 1] << 7);
            }
            // write data to slave            
            Wire.beginTransmission(i2c_slave_addr);  // transmit to slave device
            for(byte i=0;i < i2c_req_size;i++) Wire.write(i2c_req_buff[i]);
            Wire.endTransmission(true);
            break;

        case I2C_READ_QUERY:
            i2c_slave_addr = argv[0];
            read_cnt = argv[1];
            //  read data from slave
            if(read_cnt > 0){
                // read data
                if(Wire.requestFrom((int)i2c_slave_addr,(int)read_cnt,true) == read_cnt){
                    for(byte i=0;i < read_cnt;i++) i2c_rpl_buff[i] = Wire.read();
                    i2c_rpl_size = read_cnt;
                } else {
                    i2c_rpl_size = 0; // read error
                }
            } else {
                // write only, send stop condition 
                Wire.endTransmission(true);
                i2c_rpl_size = 0;
            }
            // send FIRMATA reply packet
            replayI2CReadData();
            break;

      case I2C_WRITE_AND_READ_QUERY:
            i2c_slave_addr = argv[0];
            read_cnt = argv[1];
            // prepare write data
            i2c_req_size = (argc - 2) / 2;             
            for(byte i=0;i < i2c_req_size;i++){
                i2c_req_buff[i] = argv[2 + i * 2] + (argv[2 + i * 2 + 1] << 7);
            }
            // write data to slave            
            Wire.beginTransmission(i2c_slave_addr);  // transmit to slave device
            for(byte i=0;i < i2c_req_size;i++) Wire.write(i2c_req_buff[i]);
            //  read data from slave
            if(read_cnt > 0){
                // will be send repeated start condition 
                Wire.endTransmission(false);
                // read data
                if(Wire.requestFrom((int)i2c_slave_addr,(int)read_cnt,true) == read_cnt){
                    for(byte i=0;i < read_cnt;i++) i2c_rpl_buff[i] = Wire.read();
                    i2c_rpl_size = read_cnt;
                } else {
                    i2c_rpl_size = 0; // read error
                }
            } else {
                // write only, send stop condition 
                Wire.endTransmission(true);
                i2c_rpl_size = 0;
            }
            // send FIRMATA reply packet
            replayI2CReadData();
            break;
            
        case DIGITAL_PORTS_QUERY:
            Serial.write(START_SYSEX);
            Serial.write(DIGITAL_PORTS_REPLY);
            for (byte i=0; i<TOTAL_DIGITAL_PORTS; i++) {
                pval = readPort(i, 0xff);
                Serial.write(pval & 0x7F);        // LSB
                Serial.write((pval >> 7) & 0x7F); // MSB
            }
            Serial.write(END_SYSEX);
            break;
            
        case DIGITAL_WRITE_PIN:
            digitalWrite(argv[0],argv[1]);
            break;

        case DIGITAL_CHANGE_MASK:
            if(argv[0] < TOTAL_DIGITAL_PORTS){
                f_change_detect[argv[0]] = argv[1] + (argv[2] << 7); 
            }
            break;

        case DIGITAL_BLINK_MASK:
            if(argv[0] < TOTAL_DIGITAL_PORTS){
                pval = f_blink_pins[argv[0]]; // 変更前の MASK値を保存
                f_blink_pins[argv[0]] = argv[1] + (argv[2] << 7);
                // BLINK 指定を変化させたデジタルピンを強制的に LOW に設定する
                chg_val = pval ^ f_blink_pins[argv[0]];
                for(int i = 0;i < 8;i++){
                    if(chg_val & (1 << i)){
                        digitalWrite((argv[0] * 8) + i,LOW); // 負論理の場合には HIGH にする     
                    }
                }                
            }
            break;

        case DIGITAL_CONFIG_QUERY:
            Serial.write(START_SYSEX);
            Serial.write(DIGITAL_CONFIG_REPLY);
#ifdef  USE_SPI_CONFIG
            Serial.write(0x01);  // SPI を使用している
#else
            Serial.write(0x00);  // SPI を使用していない
#endif
            for (byte i=0; i<TOTAL_DIGITAL_PORTS; i++) {
                Serial.write(f_dio_config[i] & 0x7F);        // LSB
                Serial.write((f_dio_config[i] >> 7) & 0x7F); // MSB
            }
            Serial.write(END_SYSEX);
            break;

        case ANALOG_READ_QUERY:
            Serial.write(START_SYSEX);
            Serial.write(ANALOG_READ_REPLY);
            for (byte i=0; i<USE_ANALOG_PIN_COUNT; i++) {
                adval = analogRead(i);
                Serial.write(adval & 0x7F);        // LSB
                Serial.write((adval >> 7) & 0x7F); // MSB
            }
            Serial.write(END_SYSEX);
            break;
            
        case ANALOG_MARGIN:
            f_ad_margin[argv[0]] = argv[1] + (argv[2] << 7);
            break;
            
        case SPI_WRITE_READ_QUERY:
            // prepare write data
            spi_req_size = argc / 2;             
            for(byte i=0;i < spi_req_size;i++){
                spi_req_buff[i] = argv[i * 2] + (argv[i * 2 + 1] << 7);
            }

            // write data to slave
            spi_rpl_size = spi_write_and_receive(spi_req_buff,spi_req_size,spi_rpl_buff);

            // send FIRMATA reply packet
            replaySPIReadData();
            break;

        case SPI_READ_QUERY:
            // prepare write data
            spi_req_size = argv[0];
            for(byte i=0;i < spi_req_size;i++){
                spi_req_buff[i] = 0x0;
            }

            // write data to slave
            spi_rpl_size = spi_write_and_receive(spi_req_buff,spi_req_size,spi_rpl_buff);

            // send FIRMATA reply packet
            replaySPIReadData();
            break;
            
        case SPI_WRITE:
            // prepare write data
            spi_req_size = argc / 2;             
            for(byte i=0;i < spi_req_size;i++){
                spi_req_buff[i] = argv[i * 2] + (argv[i * 2 + 1] << 7);
            }

            // write data to slave
            spi_write_and_receive(spi_req_buff,spi_req_size,spi_rpl_buff);
            break;
            
    }
}

// FIRMATA DIGITAL_MESSAGE メッセージ受信時に I/O ポートの値を更新する
void digitalWriteCallback(byte port, int value)
{
    byte currentPinValue;
    if (port < TOTAL_DIGITAL_PORTS) {
        for(byte i=0; i<8; i++) {
            currentPinValue = (byte) value & (1 << i);
            digitalWrite(i + (port*8), currentPinValue);
        }
    }
}

// 定期的にコールして、DIGITAL ポートの変化を検出した場合に
// FIRMATA メッセージを送信する
void checkAndReportIOTask(void){
    int   port_stable = 1;
    int   change_detect = 0;
    byte  diff_bits[TOTAL_DIGITAL_PORTS];

    // read port(s)
    for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
	prev_dio[p] = current_dio[p];
    }
    // read current port(s) values
    for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
        current_dio[p] = readPort(p, 0xff);
    }    
    // prev_dio == current_dio ?  チャタリング、グリッチ除去
    for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
      if(prev_dio[p] != current_dio[p]) port_stable = 0;
    }      
    if(port_stable) {
        // リセット後に最初のサンプリング値を比較する時には現在のポート値を初期値として使う
        if (first_dio_sample) {
            first_dio_sample = 0;
            for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
	        stable_prev_dio[p] = current_dio[p];
	        stable_current_dio[p] = current_dio[p];
            }
        }
        // チャタリング、グリッチ除去後のポート値を保存      
        for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
	    stable_prev_dio[p] = stable_current_dio[p];
	    stable_current_dio[p] = current_dio[p];	
        }
	// change_detect で指定されたビット位置の値が、前回取得したポート値と違っているかを調べる
        for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
	    diff_bits[p] = ((stable_prev_dio[p] & f_change_detect[p]) ^ (stable_current_dio[p] & f_change_detect[p])) & 0xFF;
            if(diff_bits[p]) change_detect = 1;
        }
        // DIGITAL_CHANGE_DETECT メッセージ送信
	if(change_detect) {
            Serial.write(START_SYSEX);
            Serial.write(DIGITAL_CHANGE_DETECT);
            for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
                Serial.write(diff_bits[p] & 0x7F);        // LSB
                Serial.write((diff_bits[p] >> 7) & 0x7F); // MSB
                Serial.write(stable_current_dio[p] & 0x7F);        // LSB
                Serial.write((stable_current_dio[p] >> 7) & 0x7F); // MSB
            }
            Serial.write(END_SYSEX);
	}
    }
}

// x,y の差分値を計算して diff_val に返す
// リターン値は差分値の絶対値を返す
uint16_t my_abs(uint16_t x,uint16_t y,int16_t* diff_val){
	*diff_val =(int16_t)x - (int16_t)y;
	if (*diff_val >= 0) {
		return (uint16_t)*diff_val;
	} else {
		return (uint16_t)-(*diff_val);
	}
}

// 定期的にコールして、ANALOGポートの変化を検出した場合に
// FIRMATA メッセージを送信する
void checkAndReportADTask(void){
    int16_t  diff_ad[USE_ANALOG_PIN_COUNT];
    uint16_t  adc[USE_ANALOG_PIN_COUNT];
    uint16_t diff_bits = 0;

    // 最新の A/D 取得
    for (byte i=0; i<USE_ANALOG_PIN_COUNT; i++) {
        adc[i] = analogRead(i);
    }
    if (first_ad_sample) {
        // リセット後、最初の A/D サンプリングの場合には現在のA/D値を基準にして検出を監視する
        first_ad_sample = 0;
        for (byte i=0; i<USE_ANALOG_PIN_COUNT; i++) {
            stable_adc[i] = adc[i];
        }
    } else {
        // リセット後、2回目以降の A/D サンプリングの場合、
        // 前回マージン値以上変化したときから今回のA/D差分値を計算して、メッセージ送信対象となるかを調べる。
	diff_bits = 0;
        for (byte i=0; i<USE_ANALOG_PIN_COUNT; i++) {
  	    if ((my_abs(adc[i],stable_adc[i],&diff_ad[i]) >= f_ad_margin[i]) && (f_ad_margin[i])) {
  	        stable_adc[i] = adc[i];
	        diff_bits += (1 << i);
	    }
        }
        // ANALOG_UPDATE メッセージ送信
	if(diff_bits) {
            Serial.write(START_SYSEX);
            Serial.write(ANALOG_UPDATE);
            Serial.write(diff_bits & 0x7F);
            for (byte i=0; i<USE_ANALOG_PIN_COUNT; i++) {
                Serial.write(adc[i] & 0x7F);        // LSB
                Serial.write((adc[i] >> 7) & 0x7F); // MSB
            }
            Serial.write(END_SYSEX);
        }
    }	

}

// デジタルポートの入出力モードをプルグラム中で保持して、問い合わせがあった時に備える
void save_dio_config(byte pin,uint8_t mode)
{
    byte port = pin / 8;  
    byte bit_val = pin % 8;
    if(mode == OUTPUT){
        f_dio_config[port] |= (1 << bit_val);
    }
    if((mode == INPUT) || (mode == INPUT_PULLUP)){
        f_dio_config[port] &= ~(1 << bit_val);
    }
}

// デジタルポートの f_blink_pins に設定されたピンを点滅表示
void blinkIOTask(void){
    static int val = HIGH;
    if (val == HIGH){
        val = LOW;
    } else {
        val = HIGH;
    }
    for(byte i = 2;i <= 13;i++){
        if(f_blink_pins[(i / 8)] & (1 << (i % 8))){
            digitalWrite(i,val);
        }
    }
}

void setup()
{
    // Firmata ライブラリ初期設定、イベントハンドラ、ボーレート設定
    Firmata.setFirmwareNameAndVersion("SensorControlModule",2,1);
    Firmata.attach(DIGITAL_MESSAGE, digitalWriteCallback);
    Firmata.attach(START_SYSEX, sysexCallback);
    Firmata.begin(57600);

    // デジタルポートモード設定
    // pinMode()関数を使用してモードを設定した場合には、save_dio_config()を続けて
    // コールして、DIGITAL_CONFIG_REPLY　コマンドで応答する設定値も更新すること。
    // (config:A) pin#2-#13 デジタルポートを全て OUTPUT モードに初期設定する
    //for(byte i = 2;i <= 13;i++){
    //    pinMode(PIN_TO_DIGITAL(i), OUTPUT);
    //    save_dio_config(PIN_TO_DIGITAL(i), OUTPUT);
    //}
    // (config:B) pin#2-#3 のデジタルポートを INPUT_PULLUP, pin#4-#13 を OUTPUT モードに初期設定する
    for(byte i = 2;i <= 3;i++){
        pinMode(PIN_TO_DIGITAL(i), INPUT_PULLUP);  
        save_dio_config(PIN_TO_DIGITAL(i), INPUT_PULLUP);  
    }
    for(byte i = 4;i <= 13;i++){
        pinMode(PIN_TO_DIGITAL(i), OUTPUT);  
        save_dio_config(PIN_TO_DIGITAL(i), OUTPUT);  
    }
    
    // DIGITAL ポート変化チェック用の変数を初期化
    first_dio_sample = 1;
    for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
	stable_prev_dio[p] = 0;
	stable_current_dio[p] = 0;
	prev_dio[p] = 0;
	current_dio[p] = 0;
        f_change_detect[p] = 0x00; // 全てのDITAILポート中のビット変化を非検出対象とする
        f_blink_pins[p] = 0x00;    // 全てのDITAILポートのブリンクモードをクリア
    }
    
#ifdef  USE_SPI_CONFIG
    // SPIで使用するピンのデジタルポート設定値を上書きする
    pinMode(PIN_TO_DIGITAL(10), OUTPUT);  // SPI SS   = D10 (OUTPUT)
    save_dio_config(PIN_TO_DIGITAL(10), OUTPUT);  
    pinMode(PIN_TO_DIGITAL(11), OUTPUT);  // SPI MOSI = D11 (OUTPUT)
    save_dio_config(PIN_TO_DIGITAL(11), OUTPUT);  
    pinMode(PIN_TO_DIGITAL(12), INPUT);  // SPI MISO = D12 (INPUT) 
    save_dio_config(PIN_TO_DIGITAL(12), INPUT);  
    pinMode(PIN_TO_DIGITAL(13), OUTPUT);  // SPI SCK  = D13 (OUTPUT)
    save_dio_config(PIN_TO_DIGITAL(13), OUTPUT);
    // SS slave select を high にする。SSアクティブ時には Low パルスが出力される
    digitalWrite(10,HIGH);
    f_change_detect[1] &= 0x03; // SPI で使用するデジタルピン D10-D13 を変化検出対象から外す

    // SPI 通信条件設定
    SPI.begin();
    SPI.setDataMode(SPI_MODE0);
    SPI.setBitOrder(MSBFIRST);
    SPI.setClockDivider(SPI_CLOCK_DIV16);

#endif

    // ADRef の設定
    analogReference(DEFAULT);  // ADRef = Vcc
    
    // ANALOG ポート変化チェック用の変数を初期化
    first_ad_sample = 1;
    for (byte p=0; p<USE_ANALOG_PIN_COUNT; p++) {
	stable_adc[p] = 0;
        f_ad_margin[p] = 0;    // 全てのANALOG ポートの値変化を検出しない
    }
    
    // I2C 初期設定、スレーブ動作時のアドレスとイベントハンドラ設定
    Wire.begin(ARDUINO_I2C_SLAVE_ADDR);
    Wire.onReceive(i2cReceiveEvent);
    Wire.onRequest(i2cRequestEvent);
    
    // i2c スレーブコマンドのイベント送信用バッファの初期化
    i2c_event_buff_init();
    
#ifdef	LOG_MESSAGE
    sprintf(svr_log_buff,"i2c_slave_addr = 0x%2.2x",(unsigned int)ARDUINO_I2C_SLAVE_ADDR);
    Firmata.sendString(svr_log_buff);

    for (byte p=0; p<TOTAL_DIGITAL_PORTS; p++) {
        sprintf(svr_log_buff,"dio_config[%d] = 0x%2.2x",p,f_dio_config[p]);
        Firmata.sendString(svr_log_buff);
    }
#endif


    // ここからユーザーセットアップルーチンを記述する
    //
    //
    //



    
}


void loop()
{
    static unsigned long tim_i2c = 0;
    static unsigned long tim_io = 0;
    static unsigned long tim_ad = 0;
    static unsigned long tim_blink = 0;
    
    // I2Cスレーブで受け付けたイベント送信コマンドのデータをPCに送信
    if (millis() - tim_i2c > 10UL) {
        tim_i2c = millis();
        i2cEvnetTask();
    }

    // DIGITAL I/O の変化をPCに送信
    if (millis() - tim_io > 10UL) {
        tim_io = millis();
        checkAndReportIOTask();
    }

    // ANALOG I/O の変化をPCに送信
    if (millis() - tim_ad > 10UL) {
        tim_ad = millis();
        checkAndReportADTask();
    }

    // DIGITAL I/O のブリンク表示
    if (millis() - tim_blink > 700UL) {
        tim_blink = millis();
        blinkIOTask();
    }
    
    // FIRMATA メッセージ処理
    while(Firmata.available()) {
        Firmata.processInput();
    }

    // ここからユーザー処理ループを記述する
    //
    //
    //





}

