
file_id = "ARDUINO/AD_MARGIN"

--[[

●機能概要

ANALOG_UPDATE メッセージの送信対象となるアナログ変化マージン値の設定を行う

Arduino では FIRMATA プロトコル [DIGITAL_MESSAGE] をサポートした
SensorControlModule.ino スケッチプログラムを動作させてください。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名						"COM4"

channel			A/D チャンネル番号 0,1,2,3 の何れか						"0"

margin			chnnel で指定したアナログデータの変化マージン値(10進数)	
				0 から 1023 の値を指定する。0 を指定した場合には ANALOG_UPDATE
				メッセージの検出対象から外す							"10"

●リターンパラメータ

なし

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[ANALOG_MARGIN] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/04/02	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["channel"] and g_params["margin"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Firmata  ANALOG_MARGIN message 送信
----------------------------------------------------------------------
local com = g_params["com"]
local firmata_command = list_to_hex(bit_or(0x90,tonumber(g_params["port"])))
local send_data = "F053" ..tbl_to_hex72({tonumber(g_params["channel"])},1) ..
				  tbl_to_hex72({tonumber(g_params["margin"])},2) .. "F7"
local stat,rdata = serial_write(com,send_data)
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

