
file_id = "ARDUINO/BLINK_MASK"

--[[

●機能概要

デジタルポートのブリンク表示を行うピンをビット値で指定する
ビットが 1 の場合に ブリンク表示を行う

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

port			Arduino digital port# 番号								"0"

mask			I/O ポートビットマスク値(16進数2桁)						"E0"

●リターンパラメータ

なし

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[DIGITAL_BLINK_MASK] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/06/03	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["port"] and g_params["mask"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- DIGITAL_BLINK_MASK 送信
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "5B"
local port_number_hexstr = list_to_hex(tonumber(g_params["port"]))
local mask_data = tbl_to_hex72({tonumber("0x" .. g_params["mask"])},2)
local send_data = sysex_start ..  firmata_request .. port_number_hexstr .. mask_data .. sysex_end

local stat,rdata = serial_write(com,send_data)
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

