
file_id = "ARDUINO/IO_GET"

--[[

●機能概要

Arduino デバイスのデジタルポート値を取得する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

PORT_0			Arduino port#0(8bit)の現在の値(16進数2桁)				"FC"
PORT_1			Arduino port#1(8bit)の現在の値(16進数2桁)				"00"
..
PORT_n			Arduino port#n(8bit)の現在の値(16進数2桁)				"00"


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[DIGITAL_PORTS_QUERY] [DIGITAL_PORTS_REPLY] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。


ATmega328 を使用した Arduino ボードの場合には、 digital I/O port は 0,1 の2 つで
それぞれ、port0:PORTD, port1:PORTB(bit#0-bit#5) + (PORTC bit#0-#1)に対応しています。
詳しくは、Arduino IDE に同梱されている pins_arduino.h ファイルを参照して下さい。

●変更履歴

2017/06/28	abs_agent 用に排他制御コール部分を変更

2014/04/22	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Arduino digital I/O ポート取得 QUERY 送信と REPLY 受信
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "61"
local firmata_reply = "62"
local send_data = sysex_start .. firmata_request .. sysex_end

----------------------------------------------------------------------
-- 排他制御開始
-- 同じ Arduino デバイスに対して、QUERY コマンドが複数同時実行される場合に備えています。
-- 別のArduino デバイスを操作する場合や、同じ Arduino デバイスに対して異なる FIRMATA REPLY
-- コマンドバイト値を取得する場合には排他制御の必要はありません
----------------------------------------------------------------------
--local cstat,handle = critical_section_enter("Firmata" .. firmata_request,5000);
local cstat = critical_section_enter2("Firmata" .. firmata_request,5000);
if not cstat then
	log_msg("critical_section_enter() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- リクエストフレーム送信とリプライ受信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data,firmata_reply)

----------------------------------------------------------------------
-- 排他制御終了
----------------------------------------------------------------------
--cstat = critical_section_leave(handle)
cstat = critical_section_leave2("Firmata" .. firmata_request)
if not cstat then
	log_msg("critical_section_leave() failed",file_id)
	error()
end
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- 受信した FIRMATA フレームの内容をリターンパラメータに格納
----------------------------------------------------------------------
local data = hex72_to_tbl(string.sub(rdata,5,-3),2)
if data ~= nil then
	for key,val in ipairs(data) do
	 	if not script_result(g_taskid,"PORT_" .. tostring(key - 1),bit_tohex(val,2)) then error() end
	end
end

