
file_id = "ARDUINO/IO_PUT"

--[[

●機能概要

Arduino デバイスのデジタルポート値を設定する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

port			Arduino port# 番号										"1"

value			I/O ポートに設定する値(16進数2桁)						"FC"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------


●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[DIGITAL_MESSAGE] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

ATmega328 を使用した Arduino ボードの場合には、 digital I/O port は 0,1 の2 つで
それぞれ、port0:PORTD, port1:PORTB(bit#0-bit#5) + (PORTC bit#0-#1)に対応しています。
詳しくは、Arduino IDE に同梱されている pins_arduino.h ファイルを参照して下さい。

●変更履歴

2014/04/22	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["port"] and g_params["value"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Firmata digital I/O message 送信
----------------------------------------------------------------------
local com = g_params["com"]
local firmata_command = list_to_hex(bit_or(0x90,tonumber(g_params["port"])))
local send_data = firmata_command .. tbl_to_hex72({tonumber("0x" .. g_params["value"])},2)
local stat,rdata = serial_write(com,send_data)
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

