
file_id = "ARDUINO/LIST"

--[[

●機能概要

シリアルデバイス管理モジュールに登録された Arduino シリアルデバイスリストを取得する。
"デバイスタイプ" が "FIRMATA" のものだけを対象にする

"タイトル" が登録されていた場合には、"タイトル" の内容がリターン値にセットされる。
"タイトル" が未登録の場合には変わりに "COMポート名" が値にセットされる。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

1				シリアルデバイス#1 "タイトル" または "COMポート名"	"Arduino1"

2				シリアルデバイス#2 "タイトル" または "COMポート名"	"COM4"

..
..

N				シリアルデバイス#N "タイトル" または "COMポート名"	"COM10"

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[ANALOG_READ_QUERY] [ANALOG_READ_REPLY] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/04/22	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]


----------------------------------------------
-- シリアルデバイスリストをマスターから取得
----------------------------------------------

local stat,com,title,type,buff = serial_all_list()
if not stat then error() end

-----------------------------------------------------------------
-- FIRMATA タイプのシリアルデバイスリストを作成
-----------------------------------------------------------------
local new_list = {}
for key,val in ipairs(type) do
	if val == "FIRMATA" then
		if title[key] ~= "" then
			table.insert(new_list,title[key])
		else
			table.insert(new_list,com[key])
		end
	end
end

---------------------------------------------
-- デバイスリストをリターンパラメータに格納
---------------------------------------------
for key,val in ipairs(new_list) do
	script_result(g_taskid,tostring(key),val)
end

