
file_id = "ARDUINO/PIN_SET"

--[[

●機能概要

Arduino デバイスの指定したデジタルピンの値を設定する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

pin			Arduino pin番号(2 から 13 までの整数)						"13"

value		pin に指定したI/Oポートに設定する値(0 または 1)				"1"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------



●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[DIGITAL_WRITE_PIN]をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2014/04/23	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["pin"] and g_params["value"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Arduino digital I/O ポート取得 QUERY 送信
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "63"
local pin = bit_tohex(tonumber(g_params["pin"]),2) 
local val = bit_tohex(tonumber(g_params["value"]),2) 
local send_data = sysex_start .. firmata_request .. pin .. val .. sysex_end

----------------------------------------------------------------------
-- リクエストフレーム送信
----------------------------------------------------------------------
local stat = serial_write(com,send_data)
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end

