
file_id = "ARDUINO/SPI_WRITE_READ"

--[[

●機能概要

Arduino SPIチャンネルに接続したスレーブデバイスにデータを書き込んだ後、
指定バイト数分のデータ読み込みを行う


●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				Arduino が接続された COM ポート名またはタイトル名		"COM4"

data			スレーブデバイスに書き込むデータ(16進数)列				"0A0B0C"
				最大31 バイトまで

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

SPI_DATA		スレーブで倍しから読み込んだデータ(16進数)列			
				最大31バイトまで										"0A0B0C

●備考

com パラメータに指定するシリアルポートは、DeviceServer の "サーバー設定プログラム"
を使用して、SERIAL サービスモジュールに FIRMATA デバイスタイプで登録したものを指定
します。

com ポートで指定したシリアルポートに接続する Arduino では、FIRMATA プロトコル 
[SPI_WRITE_READ_QUERY] [SPI_READ_REPLY] をサポートした SensorControlModule.ino 
スケッチプログラムを動作させてください。

●変更履歴

2017/06/28	abs_agent 用に排他制御コール部分を変更

2014/04/23	初版作成

ABS-9000 DeviceServer        copyright(c) All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
if not (g_params["com"] and g_params["data"]) then
	log_msg("parameter error",file_id)
	error()
end

----------------------------------------------------------------------
-- Firmata SP_WRITE_READ_QUERY リクエストフレーム作成
----------------------------------------------------------------------
local com = g_params["com"]
local sysex_start = "F0"
local sysex_end = "F7"
local firmata_request = "58"
local firmata_reply = "59"
local data_hexstr = tbl_to_hex72(hex_to_tbl(g_params["data"]),2)
local send_data = sysex_start ..  firmata_request .. data_hexstr .. sysex_end

----------------------------------------------------------------------
-- 排他制御開始
-- 同じ Arduino デバイスに対して、QUERY コマンドが複数同時実行される場合に備えています。
-- 別のArduino デバイスを操作する場合や、同じ Arduino デバイスに対して異なる FIRMATA REPLY
-- コマンドバイト値を取得する場合には排他制御の必要はありません
----------------------------------------------------------------------
-- local cstat,handle = critical_section_enter("Firmata" .. firmata_request,5000);
local cstat = critical_section_enter2("Firmata" .. firmata_request,5000);
if not cstat then
	log_msg("critical_section_enter() failed",file_id)
	error()
end

----------------------------------------------------------------------
-- リクエストフレーム送信とリプライ受信
----------------------------------------------------------------------
local stat,rdata = serial_write(com,send_data,firmata_reply)

----------------------------------------------------------------------
-- 排他制御終了
----------------------------------------------------------------------
-- cstat = critical_section_leave(handle)
cstat = critical_section_leave2("Firmata" .. firmata_request)
if not cstat then
	log_msg("critical_section_leave() failed",file_id)
	error()
end
if not stat then
	log_msg("serial_write() failed",file_id)
	error()
end


---------------------------------------------------------------
-- SYSEX コマンド I2C_READ_REPLY 受信パケット処理
---------------------------------------------------------------
local msg = ""	
local tmp = string.sub(rdata,5,-3);
if tmp ~= "" then
	local data = hex72_to_tbl(tmp,2)
	if data ~= nil then
		for key,val in ipairs(data) do
			msg = msg .. bit_tohex(val,2)
		end
	end
	if not script_result(g_taskid,"SPI_DATA",msg) then error() end
end



