
--[[

●機能概要

グローバル共有データとグローバル共有文字列リストデータをファイルにバックアップする。
バックアップしたファイルからデータをリストア(ロード)することもできる。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
mode			動作モードを指定する。パラメータ省略時はバックアップモードになる。

　　			"backup"  バックアップファイルを作成
				"restore" バックアップファイルからデータをロード

																"backup"

folder			バックアップファイルを作成するディレクトリを指定
				省略時には abs_agent プログラムがインストールされた
				ディレクトリになる。
																"/backup"


●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------
STATUS			実行結果ステータス
				"OK" または エラーメッセージ					"OK"


●備考

バックアップとリストアの機能は abs_agent のクライアントプログラム agent_data と agent_strlist
を使用しています。各クライアントプログラムの詳しい仕様については abs_agent ユーザーマニュアルを
ご覧ください。

リストア時の動作について：

  ＊グローバル共有変数は現在 abs_agent に登録済みのデータに更新を加える形で、
    バックアップファイルからデータをロードします。

  ＊グローバル共有文字列リストは既存の文字列リストをクリアしてからデータをロードします。
    クリア対象となる文字列リストは、バックアップファイル中に存在する文字列リストのみが
	対象で、その他の現在 abs_agent に登録済みの文字列リストには影響しません。


●変更履歴

2017/01/05    初版作成

ABS-9000 abs_agent copyright(c) All Blue System

]]

-----------------------------
-- パラメータ取得
-----------------------------
local stat,mod_stat,config = service_module_status()
local backup_directory = config["ProgramFolder"]
if g_params["folder"] then
	backup_directory = g_params["folder"]
end

local agent_strlist_cmd = config["ProgramFolder"] .. "/agent_strlist"
local agent_data_cmd = config["ProgramFolder"] .. "/agent_data"
local data_backup_file = backup_directory .. "/_backup_data.txt"
local strlist_backup_file = backup_directory .. "/_backup_strlist.txt"
local mode = "backup"
if g_params["mode"] then
	mode = g_params["mode"]
end
if (mode ~= "backup") and (mode ~= "restore") then error() end

-----------------------------
-- バックアップ
-----------------------------
if mode == "backup" then
	if not os.execute(agent_data_cmd .. ' -f ' .. data_backup_file) then
		script_result(g_taskid,"STATUS","*ERROR* " .. agent_data_cmd .. " failed.")
		return
	end

	if not os.execute(agent_strlist_cmd .. ' -a -f ' .. strlist_backup_file) then
		script_result(g_taskid,"STATUS","*ERROR* " .. agent_strlist_cmd .. " failed.")
		return
	end
	script_result(g_taskid,"STATUS","OK")
	return
end 

-----------------------------
-- リストア
-----------------------------
if mode == "restore" then
	if not os.execute(agent_data_cmd .. ' -p ' .. data_backup_file) then
		script_result(g_taskid,"STATUS","*ERROR* " .. agent_data_cmd .. " failed.")
		return
	end

	-- 文字列リストのリストア時に既存の文字列リストをクリアしたくない場合には、"-q" オプション指定を外して下さい
	if not os.execute(agent_strlist_cmd .. ' -q -p ' .. strlist_backup_file) then
		script_result(g_taskid,"STATUS","*ERROR* " .. agent_strlist_cmd .. " failed.")
		return
	end
	script_result(g_taskid,"STATUS","OK")
	return
end 

