
--[[

●機能概要

FASTDB データベースで使用中のキー名リストを JSON フォーマットで取得する。

●リクエストパラメータ

なし


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
KeyList			FASTDBデータベースで使用中のキー名リストがJSON フォーマット
				文字列で格納される

[
  {"Key":"<FASTDBデータベース登録時のキー#1>"},
  {"Key":"<FASTDBデータベース登録時のキー#2>"},
  ..
  ..
  {"Key":"<FASTDBデータベース登録時のキー#n>"}
]

値の例

[
 {"Key":"SENSOR_IR_Device4"},
 {"Key":"SENSOR_IR_Node1"},
 {"Key":"SENSOR_LUMI_Node1"},
 {"Key":"SENSOR_TP_Device4"},
 {"Key":"SENSOR_TP_Node1"}
]

●備考

Web API 経由でこのスクリプトを実行するときに、URL パラメータに noquote=1 を指定
して、リターンパラメータの値を直接 JSON オブジェクトとしてアクセスします。

JavaScriptから以下のURL をコールしてJSON リプライデータを受信します。

http://<hostname>:<port>/command/json/script?session=<session_token>&name=SUMMARY%2FLIST_JSON&noquote=1

取得した JSON データからデバイスリスト中の各データ項目にアクセスするときには下記の様な
JavaScript を記述します。このとき、data 変数には WebAPI で取得した JSON オブジェクトが格納されている
ものとします。

for(i in data.ResultParams.KeyList){
	..
	..  data.ResultParams.KeyList[i].Key ..
	..
}

●変更履歴

2017/4/23	abs_agent FASTDB 用に移植

2014/07/13	初版作成

abs_agent 2014-2017 copyright(c) All Blue System

]]

-----------------------------------------------------
-- 統計データベースで使用中のキー名リストを取得
-----------------------------------------------------
local stat,keys,rec_list = fastdb_key_list()
if not stat then error() end

-----------------------------------------------------------------
-- キー名リストを JSON 文字列に変換
-----------------------------------------------------------------
local key_list = "["
local cnt = 0
for key,val in ipairs(keys) do
	if rec_list[key] > 0 then
		if cnt ~= 0 then
			key_list = key_list .. ","
		end
		key_list = key_list .. '{"Key":"' .. val .. '"}'
		cnt = cnt + 1
	end
end
key_list = key_list .. "]"

---------------------------------------------
-- デバイスリストをリターンパラメータに格納
---------------------------------------------
script_result(g_taskid,"KeyList",key_list)

