
--[[

●機能概要


I2C バスに接続した温度・湿度センサー(DHT12) の値を取得する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------

node			操作対象のMGCPデバイスノード名					"ESP32Node1"

bus             I2C バス番号                                     "1"
                "0" または "1"を指定、省略時は "0" を使用する

●リターンパラメータ

---------------------------------------------------------------------------------
キー値          値                                               値の例
---------------------------------------------------------------------------------

temperature     センサーから取得した摂氏温度                     "12.5"
                                                                 "-25.0"

humidity		センサーから取得した相対湿度(%)                  "75.0"



●備考


●使用条件・免責事項

このスクリプトファイルは自由にお客様が複製、改変を行うことができます。またお客様のアプリケーションと
共にこのファイルを配布することができます。

このスクリプトは公開されているデバイス仕様書を元に、オールブルーシステムがサンプル実装したものです。
お客様がこのファイルを利用される場合には、お客様または第三者に損害が生じた場合でも、
オールブルーシステムは損害賠償その他一切の責任を負担しません。


●変更履歴

2019/10/05  初版作成

abs_agent	copyright(c) All Blue System


]]

local slave_addr = "5C"
local bus = 0
local node

-------------------------
-- パラメータチェック
-------------------------
if g_params["node"] then
	node = g_params["node"]
else
	error()
end

if g_params["bus"] then
    bus = tonumber(g_params["bus"])
end

-----------------------------------------------------------------------
-- DHT12レジスタの値を取得する
-----------------------------------------------------------------------
local stat,result = mgcp_i2c_write_read(node,bus,slave_addr,"00",5)
if not stat then error() end

---------------------------------------
-- レジスタ値から摂氏温度,湿度を計算する
---------------------------------------
local reg = hex_to_tbl(result)
if bit_and((reg[1] + reg[2] + reg[3] + reg[4]),0xff) ~= reg[5] then
	log_msg("checksum error",g_script)
	error()
end

local humidity = reg[1] + (reg[2] * 0.1)
local temperature = reg[3] + (bit_and(reg[4],0x7F) * 0.1)
if bit_and(reg[4],0x80) ~= 0x0 then
	temperature = -1 * temperature
end

script_result(g_taskid,"temperature",string.format("%3.1f",temperature))
script_result(g_taskid,"humidity",string.format("%5.1f",humidity))
