
--[[

●機能概要

M5Stick-C デバイス内蔵の LED を On/Off/Blink にする。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		操作対象のMGCPデバイス(M5Stick-C)ノード名					"STICK001"

state		LED 状態を指定する。パラメータ省略時は "off" になる

			"off"
			"on"
			"blink"


●リターンパラメータ

無し

●備考

このスクリプトを MQTT_PUBLISH イベントハンドラからコールする場合には、
script_fork_exec() 等を使用して別スレッドで実行するようにしてください。
これは MQTT PUBLISH イベントメッセージの処理をメッセージ到着順にシーケンシャルに
行う為です。詳しくは abs_agent ユーザーマニュアルを参照してください。

●変更履歴

2021/1/6	初版作成

abs_agent	copyright(c) 2021 All Blue System

]]

-------------------------
-- パラメータチェック
-------------------------
local p_node
local p_state

if g_params["node"] then
	p_node = g_params["node"]
else
	error()
end

if g_params["state"] then
	p_state = g_params["state"]
else
	p_state = "off"
end

-- M5Stick-C内蔵 LED状態変更
local cmd = {}

if p_state == "off" then
	cmd["command"] = "gpio_blink"
	cmd["gpio"] = 10
	cmd["flag"] = "off"
	cmd["level"] = 1
	stat_check(mgcp_command(p_node,g_json.encode(cmd)))
end

if p_state == "on" then
	cmd["command"] = "gpio_blink"
	cmd["gpio"] = 10
	cmd["flag"] = "off"
	cmd["level"] = 0
	stat_check(mgcp_command(p_node,g_json.encode(cmd)))
end

if p_state == "blink" then
	cmd["command"] = "gpio_blink"
	cmd["gpio"] = 10
	cmd["flag"] = "on"
	stat_check(mgcp_command(p_node,g_json.encode(cmd)))
end
