
--[[

●機能概要

MQTT デバイスのダイレクトコマンド送信とリプライ受信を行います。

preload/012_MQTT_MGCP/DXCMD_LIB.lua ファイル内で定義された mgcp_dx_command() 関数を内部で
コールします。詳しい仕様は DXCMD_LIB.lua ファイル内のコメントをご覧ください。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		リクエストコマンドの送信対象のノード名						"ESP32Node1"

request		リクエストコマンドのコマンドとパラメータ部分の16進数文字列

																		"0F"

noreply		リプライメッセージを受信しない。値は何を指定しても構わない  "1"
			(オプションパラメータ)

timeout		リプライメッセージ受信までのタイムアウト時間を指定(ms)		1000
			(オプションパラメータ)


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

Stauts			処理が成功した場合には "OK" が入る
				指定したサービスが提供されていない等で、サーバーから結果を正常に
				受け取れなかった場合にはエラーメッセージが入る
																		"OK"

reply 			サーバーから返されたリターンステータスとデータ部分の16進数文字列

																		"01"


●変更履歴

2020/8/3	初版作成

abs_agent copyright(c) All Blue System

]]

local noreply = false
local timeout = 1000
local stat

-------------------------
-- パラメータチェック
-------------------------

if g_params["noreply"] then
    noreply = true
end

if g_params["timeout"] then
    timeout = tonumber(g_params["timeout"])
end

if not g_params["node"] then
	script_result(g_taskid,"Status","*ERROR* node parameter error")
	return
end

if not g_params["request"] then
	script_result(g_taskid,"Status","*ERROR* request parameter error")
	return
end

--------------------------------
-- MGCP ダイレクトコマンド実行
--------------------------------
local stat,reply = mgcp_dx_command(g_params["node"],hex_to_tbl(g_params["request"]),noreply,timeout)
if stat then
	script_result(g_taskid,"Status","OK")
	if reply then script_result(g_taskid,"reply",tbl_to_hex(reply)) end
else
	script_result(g_taskid,"Status","*ERROR* mgcp_dx_command() failed.")
end

