
--[[

●機能概要

MQTT ブローカを利用したMGCP プロトコルでイベントメッセージを送信します。

preload/012_MQTT_MGCP/MGCP_LIB.lua ファイル内で定義された mgcp_event() 関数を内部で
コールします。詳しい仕様は MGCP_LIB.lua ファイル内のコメントをご覧ください。


●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		リクエストコマンドの送信対象のノード名						"ESP32Node1"

event		イベントメッセージ(JSONフォーマット) 文字列
																		{"gpio":23,"direction":"rise"}

type		イベントタイプ文字列
																		"gpio_change_detect"

application	イベント送信元アプリケーション
			パラメータ省略時は "abs_agent" が使用される					"abs_agent"


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

Stauts			処理が成功した場合には "OK" が入る
				サーバーから結果を正常に受け取れなかった場合にはエラーメッセージが入る
																		"OK"


●備考

このスクリプトを使用する前に、abs_agent の設定ファイルに MQTTブローカへの
エンドポイントを作成しておきます。また、MQTT PUBLISH メッセージを受信した場合に
実行される MQTT_PUBLISH イベントハンドラ中に、リプライメッセージ
受信時の処理を記述しておきます。詳しくは、MQTT_PUBLISH イベントハンドラ中の
コメントアウトされた処理部分をご覧ください。

このスクリプトで使用している MQTT ブローカエンドポイントの設定例です。
abs_agent.xml 設定ファイル中の MQTT 設定に記述します。詳しいタグの説明は
abs_agent ユーザーマニュアルを参照してください。

<ClientID> と <BrokerHostName> は使用する MQTT ブローカに合わせて下さい。

    <MQTT>
      <AutoOnline type="boolean">True</AutoOnline>
      <KeepAliveTimer type="integer">60</KeepAliveTimer>
      <EndPointList>
        <Item>
          <Title>リモートデバイス処理</Title>
          <ClientID>abs9k:12345-raspi</ClientID>
          <BrokerHostName>192.168.100.14</BrokerHostName>
          <PortNumber>1883</PortNumber>
          <AutoSubscribeTopicList>/reply/#</AutoSubscribeTopicList>
          <AutoSubscribeQoSList>0</AutoSubscribeQoSList>
          <UserName/>
          <Password/>
          <WillTopic/>
          <WillMessage/>
          <WillQoS>0</WillQoS>
          <WillRetain>False</WillRetain>
          <RecvBuffInit>2000</RecvBuffInit>
          <DetailLog>True</DetailLog>
        </Item>
      </EndPointList>
    </MQTT>


●変更履歴

2018/07/30  mgcp_event() に対応して endpoint パラメータを自動で取得するように変更

2018/07/09  endpoint パラメータのデフォルト値をグローバル共有変数から取得できるようにした

2018/07/04	初版作成

abs_agent copyright(c) All Blue System

]]

local application = "abs_agent"
local stat

-------------------------
-- パラメータチェック
-------------------------
if g_params["application"] then
    application = g_params["application"]
end

if not g_params["node"] then
	script_result(g_taskid,"Status","*ERROR* node parameter error")
	return
end

if not g_params["event"] then
	script_result(g_taskid,"Status","*ERROR* event parameter error")
	return
end

if not g_params["type"] then
	script_result(g_taskid,"Status","*ERROR* type parameter error")
	return
end

-------------------------
-- MGCP イベント送信
-------------------------
local stat = mgcp_event(g_params["node"],g_params["type"],g_params["event"],application)
if stat then
	script_result(g_taskid,"Status","OK")
else
	script_result(g_taskid,"Status","*ERROR* mgcp_event() failed.")
end

