
--[[

●機能概要

MGCP_GPIO ノードの現在の GPIO ステータス値を取得してログに出力する。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

node		リクエストコマンドの送信対象のノード名						"ESP32Node1"

timeout		リプライメッセージ受信までのタイムアウト時間を指定(ms)		6000
			(オプションパラメータ)


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------

Stauts			処理が成功した場合には "OK" が入る
				指定したサービスが提供されていない等で、サーバーから結果を正常に
				受け取れなかった場合にはエラーメッセージが入る
																		"OK"


●実行例(ログサーバーに出力)

-------------------------------------------
 GPIO    port_val change_detect blink
-------------------------------------------
GPIO#00      1        0           0  
GPIO#01      1        0           0  
GPIO#02      0        0           0  
GPIO#03      1        0           0  
GPIO#04      0        0           0  
GPIO#05      1        0           0  
GPIO#06      0        0           0  
GPIO#07      1        0           0  
GPIO#08      0        0           0  
GPIO#09      1        0           0  
GPIO#10      1        0           0  
GPIO#11      1        0           0  
GPIO#12      0        0           0  
GPIO#13      1        1           0  
GPIO#14      1        0           0  
GPIO#15      1        0           0  
GPIO#16      0        0           1  
GPIO#17      0        1           0  
GPIO#18      0        0           0  
GPIO#19      0        0           0  
GPIO#20      0        0           0  
GPIO#21      0        0           0  
GPIO#22      0        0           0  
GPIO#23      0        0           0  
GPIO#24      0        0           0  
GPIO#25      0        0           0  
GPIO#26      0        0           0  
GPIO#27      1        0           1  
GPIO#28      0        0           0  
GPIO#29      0        0           0  
GPIO#30      0        0           0  
GPIO#31      0        0           0  
GPIO#32      0        0           0  
GPIO#33      0        0           0  
GPIO#34      0        0           0  
GPIO#35      1        0           0  
GPIO#36      0        0           0  
GPIO#37      0        0           0  
GPIO#38      0        0           0  
GPIO#39      0        0           0  


●変更履歴

2018/07/30  mgcp_command() に対応して endpoint パラメータを自動で取得するように変更

2017/3/5	初版作成

abs_agent copyright(c) All Blue System

]]

local timeout = 2000

-------------------------
-- パラメータチェック
-------------------------
if g_params["timeout"] then
    timeout = tonumber(g_params["timeout"])
end

if not g_params["node"] then
	script_result(g_taskid,"Status","*ERROR* node parameter error")
	return
end

-------------------------
-- MGCP コマンド実行
-------------------------
local stat,reply = mgcp_command(g_params["node"],'{"command":"gpio_get_all"}',false,timeout)
if not stat then
	script_result(g_taskid,"Status","*ERROR* mgcp_command() failed.")
	return
end

------------------------------------------------------------
-- MGCP リプライメッセージ(JSON)のデコードとエラーチェック
------------------------------------------------------------
local node_stat = g_json.decode(reply)
if node_stat.result ~= "success" then
	script_result(g_taskid,"Status","*ERROR* " .. node_stat.error_text)
	return
end

------------------------------
-- GPIO ステータスをログに出力
------------------------------
local val_bits = hex_to_bit(node_stat.gpio_val)
local change_detect_bits = hex_to_bit(node_stat.change_detect)
local blink_bits = hex_to_bit(node_stat.gpio_blink)

log_msg("-------------------------------------------")
log_msg(" GPIO    port_val change_detect blink")
log_msg("-------------------------------------------")
for key = 0,#val_bits,1 do
	log_msg(string.format("GPIO#%2.2d      %1.1d        %1.1d           %1.1d  ",
			key,val_bits[key],change_detect_bits[key],blink_bits[key]))
	wait_time(10)
end
script_result(g_taskid,"Status","OK")

