
--[[

●機能概要

MGCP ノードリスト一覧を JSON フォーマットで取得する。

●リクエストパラメータ

なし


●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
NodeList		heartbeat イベント送信元のMGCPノード名リストがJSON フォーマット文字列で格納される

[
  {"Node":"<ノード名#1>"},
  {"Node":"<ノード名#2>"},
  ..
  ..
  {"Node":"<ノード名#n>"}
]

値の例

[
 {"Node":"STICK001"},
 {"Node":"ESP32_N01"},
 {"Node":"M5_N001"}
]

●備考

Web API 経由でこのスクリプトを実行するときに、URL パラメータに noquote=1 を指定
して、リターンパラメータの値を直接 JSON オブジェクトとしてアクセスします。

JavaScriptから以下のURL をコールしてJSON リプライデータを受信します。

http://<hostname>:<port>/command/json/script?session=<session_token>&name=MGCP%2NODE_LIST_JSON&noquote=1

取得した JSON データからデバイスリスト中の各データ項目にアクセスするときには下記の様な
JavaScript を記述します。このとき、data 変数には WebAPI で取得した JSON オブジェクトが格納されている
ものとします。

for(i in data.ResultParams.NodeList){
	..
	..  data.ResultParams.NodeList[i].Key ..
	..
}

●変更履歴

2021/06/16	初版作成

abs_agent 2021 copyright(c) All Blue System

]]

-----------------------------------------------------
-- MGCP ノード名リストを取得
-----------------------------------------------------
local keys = stat_check(get_shared_strlist('$MGCP_NodeList'))

-----------------------------------------------------------------
-- キー名リストを JSON 文字列に変換
-----------------------------------------------------------------
local key_list = "["
local cnt = 0
for key,val in ipairs(keys) do
	if cnt ~= 0 then
		key_list = key_list .. ","
	end
	key_list = key_list .. '{"Node":"' .. val .. '"}'
	cnt = cnt + 1
end
key_list = key_list .. "]"

---------------------------------------------
-- デバイスリストをリターンパラメータに格納
---------------------------------------------
script_result("NodeList",key_list)

