
file_id = "MQTT_CONNECT_ALL"

--[[

●機能概要

登録された全ての MQTT エンドポイントリストの接続を開始する。


●リクエストパラメータ
---------------------------------------------------------------------------------
キー値            値                                                        値の例
---------------------------------------------------------------------------------

●リターンパラメータ
---------------------------------------------------------------------------------
キー値            値                                                        値の例
---------------------------------------------------------------------------------

●備考

サーバー設定プログラムで登録された MQTT エンドポイントに対して以下の動作を行います。

mqtt_open() を使用してソケット接続
mqtt_connect() を使用してブローカに接続
mqtt_subscribe() を使用して自動購読に指定したトピックを購読


●変更履歴

2016/06/01  abs_agent 用に移植

2015/05/13  初版作成 for ABS-9000 DeviceServer

]]

----------------------------------------------
-- 最新のエンドポイントリストを取得
----------------------------------------------
local stat, enable, ready, title, id, host, port, subscribe_topic_list, subscribe_qos_list,
    user, password, will_topic, will_message, will_qos, will_retain, recv_buff_size,log = mqtt_all_list()
if not stat then error() end

----------------------------------------------
-- エンドポイントをブローカに接続
----------------------------------------------
for key,val in ipairs(id) do

    ---------------------------------------------------------
    -- 現在ソケット接続していないエンドポイントを対象にする
    ---------------------------------------------------------
    if (not enable[key]) then

        ------------------------------
        -- Broker にソケット接続開始
        ------------------------------
        if mqtt_open(id[key]) then

            ------------------------------------
            -- Broker に CONNECT リクエスト送信
            ------------------------------------
            local cstat,connack = mqtt_connect(id[key])
            if cstat then
                if (connack == 0) then
                    if subscribe_topic_list[key] ~= "" then
                        -----------------------
                        -- デフォルトTopic購読
                        -----------------------
                        mqtt_subscribe(id[key])
                    end
                else
                    ----------------------------------------------------
                    -- Brokerから CONNECT を拒否された時はソケット削除
                    ----------------------------------------------------
                    log_msg("*WARNING* connack is not 0, closing socket " .. id[key],file_id)
                    mqtt_close(id[key])
                end
            end
        end
    end
end

