
--[[

******************************************************************************

PERIODIC_TIMER イベントハンドラスクリプトは約 1 分に１回自動的に実行されます。

一つのスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。

また、同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が
待たされる原因にもなります。

頻繁には発生しない条件で、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************

]]

---------------------------------------------------------------------------------------
-- リモートアクセス時の接続エラーを保持しているエラーフラグを一定期間過ぎたらクリアする
---------------------------------------------------------------------------------------
if not script_exec("REMOTE_LOST_HOSTS_RESET","","") then error() end

------------------------------------------------------------------
-- FASTDB データベースから過去のデータを削除する(8時間に一回実行)
------------------------------------------------------------------
local timer = "$FASTDB_PURGE_TIMER"
local val = stat_check(inc_shared_data(timer))
if (tonumber(val) >= 480) then
	stat_check(set_shared_data(timer,""))
	if not script_fork_exec("FASTDB_PURGE","","") then error() end
end

---------------------------------------------------------------------
-- FASTDB で排他ロック中のキーが一定期間以上存在しつづけている場合に、
-- 強制的にロック状態を解除する
---------------------------------------------------------------------
if not script_fork_exec("FASTDB_FREE_LOCK","lock_timeout","60") then error() end

------------------------------------------------------------------------------------------------
-- TCPClientList で保持しているクライアント接続の中で最後に使用してから時間が経過したものを削除
------------------------------------------------------------------------------------------------
if not script_fork_exec("TCPCLIENT_PURGE","","") then error() end

---------------------------------------------------------------
-- OS のステータス情報を取得して FASTDB に格納(5分に一回実行)
---------------------------------------------------------------
local timer = "$STORE_OS_STATUS_TIMER"
local val = stat_check(inc_shared_data(timer))
if (tonumber(val) >= 5) then
	stat_check(set_shared_data(timer,""))
	local CPUSTAT = stat_check(script_exec2("OS/GET_STATUS","",""))
	--if not fastdb_add("CPULoad",tonumber(CPUSTAT["CPULoad"])) then error() end
	if not fastdb_add("FreeMemMB",tonumber(CPUSTAT["FreeMemMB"])) then error() end
	if not fastdb_add("FreeDiskGB",tonumber(CPUSTAT["FreeDiskGB"])) then error() end
end
