
--[[

●機能概要

このスクリプトをコールした回数が一定数超えるとリモートエラーフラグ(文字列リスト)からホスト名を削除する

●リクエストパラメータ
---------------------------------------------------------------------------------
キー値			値		    									値の例
---------------------------------------------------------------------------------
force		無条件にエラーフラグ全体(共有文字列リスト)を削除する。
			パラメータの値には何を指定しても構わない。			"1"

●リターンパラメータ
---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
なし

●参照するグローバル共有データ

---------------------------------------------------------------------------------
キー値          値              
---------------------------------------------------------------------------------
$REMOTE_LOST_HOSTS_CLEAR_TIME											"10" 
				エラーフラグをクリアするまでのカウンタ初期値
				グローバル共有データが未設定の場合には "10" (10分) を使用

●備考

スクリプト中の init_cntr 変数に指定する値によって、リモートエラーフラグをクリアするまでの
時間を調整できます。デフォルト値は "10" で PERIODIC_TIMER スクリプトからこのスクリプトを
コールしている場合には 10 分で接続エラーフラグがクリアされます。

エラーフラグがクリアされると、以降のリモートアクセスは実際にソケット接続を試みるようになります。
エラーフラグ等についての詳しい内容はユーザーマニュアルを参照してください

●変更履歴

2021/2/13	カウンタ初期値をグローバル共有変数から取得可能にした。

2016/8/4    abs_agent 用に移植

2015/9/27	初版作成
									2021 copyright(c) All Blue System

]]

local init_cntr = stat_check(get_shared_data("$REMOTE_LOST_HOSTS_CLEAR_TIME"))
if init_cntr == "" then init_cntr = "10" end
local channel = "$REMOTE_LOST_HOSTS"
local prefix = "$NET_ERR_RST_CNTR_"
local strlist,cntr_val,new_cntr_val

---------------------------------------------------------------
-- リモートエラーフラグとカウンタ用グローバル変数を削除
---------------------------------------------------------------
if g_params["force"] then
	strlist = stat_check(get_shared_strlist(channel,false))
	for key,hostname in ipairs(strlist) do
		set_shared_data(prefix .. hostname,"")
	end
	clear_shared_strlist(channel)
end

----------------------------------------------------------------------
-- リモートエラーフラグ(文字列リスト)に登録済みのホスト名リストを取得
----------------------------------------------------------------------
strlist = stat_check(get_shared_strlist(channel,false))
for key,hostname in ipairs(strlist) do
	cntr_val = stat_check(get_shared_data(prefix .. hostname))
	if cntr_val == "" then 
		---------------------------------------------------------------------------------------
		-- カウンタ用のグローバル共有変数がまだ作成されていない場合には初期値(init_cntr)を設定
		---------------------------------------------------------------------------------------
		stat_check(set_shared_data(prefix .. hostname,init_cntr))
	else
		---------------------------------------------------------------------------------------------------
		-- ホスト名毎に作成されたカウンタをデクリメントして、0 に達した場合にはリモートエラーフラグから削除
		---------------------------------------------------------------------------------------------------
		new_cntr_val = stat_check(dec_shared_data(prefix .. hostname))
		if new_cntr_val == "" then
			stat_check(remove_shared_strlist(channel,hostname))
		end
	end
end
