
file_id = "SERIAL_ZB"

--[[

******************************************************************************
* イベントハンドラスクリプト実行時間について                                 *
******************************************************************************

一つのスクリプトの実行は長くても数秒以内で必ず終了するようにしてください。
処理に時間がかかると、イベント処理の終了を待つサーバー側でタイムアウトが発生します。

また、同時実行可能なスクリプトの数に制限があるため、他のスクリプトの実行開始が
待たされる原因にもなります。

頻繁には発生しないイベントで、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************


SERIAL_ZB スクリプト起動時に渡される追加パラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
COMPort			イベントを送信したシリアルデバイスの COMポート名		"COM3"

Title			イベントを送信したシリアルデバイスのタイトル名			"コントローラ#1"
				タイトルが設定されていない場合にはこのパラメータは
				設定されません

FRAME_DATA		XBee デバイスから取得したフレームデータ。
				(16進数、可変長)										"7E000588014E4400E4"


FRAME_DATA で渡されたフレームデータを解析して作成される文字列変数

SenderNodeID, SenderAddr64, SenderAddr16
				FRAME_DATA に格納されたフレームデータ中のペイロード部分を解析して送信元ノードのノード名と
				アドレス(16進数文字列) に変換したもの。パケットタイプが 0x90の時にのみ有効。

PayloadString
				FRAME_DATA に格納されたフレームデータ中のペイロード部分を文字列に変換したもの。
				パケットタイプが 0x90 の時にのみ有効で、文字列がペイロードに含まれていない
				場合には 空文字列が設定される。

frame_arr		
				FRAME_DATA で渡されたフレームデータ全体を数値配列(バイト列) に変換したもの


]]

-------------------------------------------------------------------------------------
-- XBee Packet のフレームデータからイベントハンドラ内で使用可能な変数を設定
-------------------------------------------------------------------------------------
local frame_arr = hex_to_tbl(g_params["FRAME_DATA"])
local PayloadString = ""
local stat,SenderAddr64, SenderAddr16
local SenderNodeID = ""
if (frame_arr[4] == 0x90) then
	SenderAddr64 = tbl_to_hex(frame_arr,5,12)
	SenderAddr16 = tbl_to_hex(frame_arr,13,14)

	local found,dev = master_item_find("ZBDevice","SerialNumber",SenderAddr64)
	if found then
		SenderNodeID = dev["NodeIdentifier"]
	end

	-- PayloadString
	local TmpArr = {}
	for i = 16,#frame_arr - 1,1 do
		if (frame_arr[i] >= 0x20) and (frame_arr[i] <= 0x7E) then
			table.insert(TmpArr,frame_arr[i])
		end
	end
	PayloadString = tbl_to_str(TmpArr)
end

------------------------------------------------------------------------------------
-- マルチフレーム受信時に zb_at_local() 関数に値を返すための文字列リストを構築する
------------------------------------------------------------------------------------
if (frame_arr[4] == 0x88) and (frame_arr[6] == 0x4E) and (frame_arr[7] == 0x44) then -- "ND" response?
	local req_flag = "ZB_STORE_REPLY_" .. g_params["COMPort"] .. "_" .. bit_tohex(frame_arr[5],2)
	local stat,flag = get_shared_data(req_flag)
	if flag ~= "" then
		add_shared_strlist("ZB_REPLY_MFRAME_" .. g_params["COMPort"] .. "_" .. bit_tohex(frame_arr[5],2),g_params["FRAME_DATA"])
	end
end

-------------------------------------------------------------------------
-- 受信したパケットのフレームデータまたはペイロード部分の文字列をログ出力
-------------------------------------------------------------------------
if PayloadString ~= "" then
	log_msg(SenderNodeID .. "[" .. g_params["COMPort"] .. "] " .. PayloadString,file_id)
else
	log_msg(SenderNodeID .. "[" .. g_params["COMPort"] .. "] " .. g_params["FRAME_DATA"],file_id)
end


--[[
*******************************************************************************************************

(1) TDCP デバイスからリプライパケットを受信したときの処理

XBee フレームを受信した時に、先頭の３バイトが ASCII で '$$$' で始まって、
次の TDCP リプライパケットに該当しない場合には TDCP イベントパケットを受信したものとして
処理を行う。
デフォルトでは、受信した文字列データをそのままログに出力するだけの処理が記述されています。


(2) TDCP デバイスからリプライパケットを受信したときの処理

共有データに、$TDCP_STORE_REPLY_<TDCP_Prefix> に何らかのデータ(ex: '1') が設定されていた場合には
リプライデータを共有データ経由でリクエスト元 xbee_tdcp_command() 関数に返す必要がある。

受信したパケット中のペイロード部分の文字列(csv) を共有変数 $TDCP_<TDCP_Prefix> に設定する。
その後、$TDCP_STORE_REPLY_<TDCP_Prefix> 共有変数を削除する。

また、データ受け渡しのタイミングを計るために下記のイベントフラグを使用する
	flag_name: $TDCP_COMMAND_<TDCP_Prefix>

例： 受信した XBee パケットのフレームデータが "$$$aa,1,1234,5678,OK" の場合、

     (1) 共有データ "$TDCP_STORE_REPLY_$$$aa" が存在する場合に以下のレコードを作成する

         Key: "$TDCP_$$$aa"    Value: "$$$aa,1,1234,5678,OK"

     (2) 共有データ "$TDCP_STORE_REPLY_$$$aa" を消去する

	 (3) イベントフラグ $TDCP_COMMAND_<TDCP_Prefix> をセットする

*******************************************************************************************************
]]

if (frame_arr[4] == 0x90) and (frame_arr[16] == 0x24) and	(frame_arr[17] == 0x24) and	(frame_arr[18] == 0x24) then
	local FrameCSVArr = csv_to_tbl(PayloadString)
	local stat,val = get_shared_data("$TDCP_STORE_REPLY_" .. FrameCSVArr[1],true)

	-- TDCP コマンドリプライを受信した場合にはリクエスト元にデータを渡す
	if val ~= '' then 
		set_shared_data("$TDCP_" .. FrameCSVArr[1],PayloadString,true) 
		event_set2x("$TDCP_COMMAND_" .. FrameCSVArr[1])
		return
	end

	-- TDCP イベントデータを受信した場合の処理
	-- log_msg(SenderNodeID .. "[" .. g_params["COMPort"] .. "] " .. PayloadString,file_id)

end

