
--[[

●機能概要

retain モードで TCPServer に接続中のクライアント接続で最終アクセス時刻が古いものを削除する
また、接続中にエラーを検出しているものも削除する

●リクエストパラメータ
												
なし

●リターンパラメータ

なし

●参照するグローバル共有データ

---------------------------------------------------------------------------------
キー値          値              
---------------------------------------------------------------------------------
$TCPCLIENT_PURGE_BEFORE											"60" 
				現在時刻からこの秒数分過去の時刻よりも古い最終アクセス時刻をもつ接続を削除する
				グローバル共有データが未設定の場合には "60" (1分) を使用

●備考


●変更履歴

2023/04/24  初版作成

abs_agent	copyright(c) 2023 All Blue System

]]

local sec_before = 60
local val = stat_check(get_shared_data("$TCPCLIENT_PURGE_BEFORE"))
if val ~= "" then sec_before = tonumber(val) end

-- 削除対象日付を取得
local year, month, day, hour, min, sec, millisec = stat_check(now_datetime())
local new_year, new_month, new_day, new_hour, new_min, new_sec = stat_check(inc_second(-1 * sec_before, year, month, day, hour, min, sec))
local timestamp = string.format("%4.4d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",new_year,new_month,new_day,new_hour,new_min,new_sec)
--log_msg("data purge until " .. timestamp)

-- 古い接続を削除
stat_check(tcp_client_purge(timestamp))
