
file_id = "TICK_TIMER"

--[[

******************************************************************************

TICK_TIMER イベントハンドラスクリプトは約 1 秒に１回自動的に実行されます。このため、
スクリプトの実行は１秒以内で必ず終了するようにしてください。

TICK_TIMER イベントハンドラは各々別スレッドで起動されています。
このため、処理時間がかかると、平行して実行されるスレッド数が増え続けて 
OS がハングアップする原因になります。

頻繁には発生しない条件で、処理時間がかかるスクリプトを実行したい場合は
スクリプトを別に作成して、このイベントハンドラ中から script_fork_exec() を使用して
別スレッドで実行することを検討してください。

******************************************************************************

]]

-- log_msg("start..",file_id)

---------------------------------------------------
-- abs_agent 起動時からのカウンター
---------------------------------------------------
local uptimer = "$SERVER_STARTUP_TIMER"
local uptime = stat_check(inc_shared_data(uptimer))

-------------------------------------------------------------------------------------------------
-- FASTDB TemporaryDB のデータ同期 
-- 起動直後の初期化タスクによるスクリプトプール逼迫を防ぐためにサーバー起動2分後から10秒毎に実行
-------------------------------------------------------------------------------------------------
local synctimer = "$FASTDB_SYNC_TIMER"
local val = stat_check(inc_shared_data(synctimer))
if (tonumber(val) >= 10) and (tonumber(uptime) >= 120) then
	stat_check(set_shared_data(synctimer,""))
	stat_check(script_fork_exec("FASTDB_SYNC",{}))
end

------------------------------------------------------------
-- 10秒に一回実行する
------------------------------------------------------------

--[[
stat,val = inc_shared_data("TIME_10S")
if not stat then error() end
if (tonumber(val) >= 10) then
	if not set_shared_data("TIME_10S","") then error() end -- counter clear

	-- チェック対象のスクリプト名を格納したテーブル
	local respawn_tasks = {}
	table.insert(respawn_tasks,"RASPI/ENVSENSOR_ALARM_TASK")
	table.insert(respawn_tasks,"RASPI/ENVSENSOR_DISPLAY_TASK")

	-- 現在実行中のタスクリストにチェック対象のスクリプトが存在しない場合には起動させる
	local stat,idlist,namelist = script_task_list()
	if not stat then error() end
	for k,v in ipairs(respawn_tasks) do
		local found = false
		for ck,cv in ipairs(namelist) do
			if cv == v then found = true end
		end
		if not found then
			log_msg("*WARNING* " .. v .. " script is not running, re-start the script..","RESPAWN_CHECK")
			script_fork_exec(v,"","")
		end
	end

end
]]

