
--[[

●機能概要

リモート XBee デバイスにデータを送信する

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値	値				           								値の例
---------------------------------------------------------------------------------
com		XBee デバイスをローカルに接続しているシリアルポート名またはタイトル名
		省略した場合には abs_agent に登録済みのシリアルデバイスを検索して
		デバイスタイプが "XBEE" のものを自動的に選択する。このとき、シリアル
		デバイスが見つからなかった場合や、2 つ以上の "XBEE" デバイスが見つかった
		場合にはエラーとなる。
																"/dev/ttyUSB0"
dest
		送信先のXBeeデバイスを指定する。dest文字列には以下のいずれかを指定する。
		XBeeデバイスの Node Identification 文字列
		XBeeデバイスの Serial Number(64bitアドレス)16進数表記の文字列
		XBeeデバイスの Source Address(16bitアドレス)16進数表記の文字列
		"*" ブロードキャストアドレス

param_hex (hexstr)
		送信データ(複数バイト)を16進数文字列で指定する。
		param_str パラメータと同時に指定してはいけない。
																"01020304AABBCCDD"

param_str (text string)
		送信データ(複数バイト)をASCII文字列で指定する。
		param_hex パラメータと同時に指定してはいけない。
																"Hello World!!"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値	値		            									値の例
---------------------------------------------------------------------------------
Stauts	処理が成功した場合には "OK" が入る
		指定したサービスが提供されていない等で、サーバーから結果を正常に
		受け取れなかった場合にはエラーメッセージが入る
		送信ステータスフレーム中のステータスバイトの値が 0x00 以外の場合にも
		エラーメッセージが格納される。
																"OK"

com		ローカル XBee デバイスが接続されたシリアルポート名		"/dev/ttyUSB0"


●備考

●変更履歴

2019/7/2	初版作成

abs_agent	copyright(c) 2019 All Blue System

]]


--------------------------------------------------------------------
-- ローカル XBee デバイスが接続されたシリアルポート名を取得する
--------------------------------------------------------------------
local com
if g_params["com"] and (g_params["com"] ~= "") then
	com = g_params["com"]
else
	-- シリアルポートを検索して XBee デバイスタイプのポート名取得を試みる
	local xbee_port_count = 0
	local stat,port,title,dev_type,buff = serial_all_list()
	if not stat then error() end
	for key,val in ipairs(port) do
		if dev_type[key] == "XBEE" then
			xbee_port_count = xbee_port_count + 1
			com = val
		end
	end
	if xbee_port_count == 0 then
		script_result("Status","*ERROR* serial(type:XBEE) port not found.")
		return
	end
	if xbee_port_count > 1 then
		script_result("Status","*ERROR* multiple serial(type:XBEE) ports.")
		return
	end
end
script_result("com",com)

--------------------------------------------------------------------
-- AT コマンド実行対象のデバイスを選択する
--------------------------------------------------------------------
local dest = ""
if g_params["dest"] and (g_params["dest"] ~= "") then
	dest = g_params["dest"]
else
	script_result("Status","*ERROR* dest parameter not found.")
	return
end

--------------------------------------------------------------------
-- 送信データを 16進数文字列に揃える
--------------------------------------------------------------------
local param = ""
if g_params["param_hex"] and (g_params["param_hex"] ~= "") then
	param = g_params["param_hex"]
end
if g_params["param_str"] and (g_params["param_str"] ~= "") then
	param = tbl_to_hex(str_to_tbl(g_params["param_str"]))
end
if param == "" then
	script_result("Status","*ERROR* TX data is null.")
	return
end

--------------------------------------------------------------------
-- AT コマンド実行とリプライ受信
--------------------------------------------------------------------
local stat,frame = xbee_transmit_data(com,dest,param)
if not stat then
	script_result("Status","*ERROR* xbee_transmit_data() failed")
	return
end
local frame_arr = hex_to_tbl(frame)
if frame_arr[6] ~= 0x00 then
	script_result("Status","*ERROR* Transmit Status, Status = 0x" .. bit_tohex(frame_arr[6],2))
	return false
end

script_result("Status","OK")
