
--[[

●機能概要

マスターに登録された XBee-ZB デバイスリストを取得する。

●リクエストパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
com				XBee-ZB デバイスをローカルに接続しているシリアルポート名またはタイトル名
				省略した場合には abs_agent に登録済みのシリアルデバイスを検索して
				デバイスタイプが "ZB" のものを自動的に選択する。このとき、シリアル
				デバイスが見つからなかった場合や、2 つ以上の "ZB" デバイスが見つかった
				場合にはエラーとなる。
																		"/dev/ttyUSB0"

●リターンパラメータ

---------------------------------------------------------------------------------
キー値			値		            									値の例
---------------------------------------------------------------------------------
Stauts			処理が成功した場合には "OK" が入る
				指定したサービスが提供されていない等で、サーバーから結果を正常に
				受け取れなかった場合にはエラーメッセージが入る
																		"OK"
com				ローカル XBee デバイスが接続されたシリアルポート名		"/dev/ttyUSB0"
count			デバイスリスト数(n)										"5"		
1			<addr64#1>,<addr16#1>,<node_id#1>,<dev_type#1>,<digi_type#1>,<is_local#1>
2			<addr64#2>,<addr16#2>,<node_id#2>,<dev_type#2>,<digi_type#2>,<is_local#2>
..
..
n			<addr64#n>,<addr16#n>,<node_id#n>,<dev_type#n>,<digi_type#n>,<is_local#n>

●備考

●変更履歴

2019/9/13	初版作成

abs_agent	copyright(c) 2019 All Blue System

]]

local com_port

if g_params["com"] and (g_params["com"] ~= "") then
	com_port = g_params["com"]
else
	-- シリアルポートを検索して XBee デバイスタイプのポート名取得を試みる
	local xbee_port_count = 0
	local stat,com,title,dev_type,buff = serial_all_list()
	if not stat then error() end
	for key,val in ipairs(com) do
		if dev_type[key] == "ZB" then
			xbee_port_count = xbee_port_count + 1
			com_port = val
		end
	end
	if xbee_port_count == 0 then
		script_result("Status","*ERROR* serial(type:ZB) port not found.")
		return
	end
	if xbee_port_count > 1 then
		script_result("Status","*ERROR* multiple serial(type:ZB) ports.")
		return
	end
end
script_result("com",com_port)


local stat,addr64,addr16,node_id,dev_type,digi_type,is_local = zb_all_list(com_port)
if not stat then
	script_result("Status","*ERROR* zb_all_list() failed")
	return
end

script_result("count",tostring(#addr64))
for i,v in ipairs(addr64) do
	script_result(tostring(i),addr64[i] .. "," .. addr16[i] .. "," .. node_id[i] .. ',"'.. dev_type[i] .. '",' .. digi_type[i] .. "," .. tostring(is_local[i]))
end

script_result("Status","OK")
