
--[[
********************************************************************
	
	EVENT2_LIB.lua for ABSAgent

	ver1.0 initial revision
	ver1.1 added event_wait_either2()
	ver1.2 use event_set_common() and event_wait_common() for ABSAgent
	ver1.3 fixed bug, event_wait2(), event_wait_either2() timeout condition not occur
	ver1.4 changed executing order ("add flags to the waiting-flag-list" <=> "add the taskid to the waiting-task-list") 
	ver1.5 mearged to native functions.

	copyright(c) 2012-2021 All Blue System 

********************************************************************
]]

-- cmn_event_flag = "$EVENT_FLAG"

--[[
cmn_event_tasklist = "$EVENT_TASKLIST"
cmn_event_flaglist_pre = "$EVENT_FLAGS_"
--]]

--[[
********************************************************************

	stat = event_wait2(flag1,flag2 ...[,timeout])

********************************************************************
]]

function event_wait2(...)
	return event_wait2x(...)
end

--[[
function event_wait2(...)
	local file_id = "event_wait2()"
	local timeout = -1
	local argv = {...}
	local argc = #argv
	local waiting_flaglist = cmn_event_flaglist_pre .. g_taskid
	local wstat
	local w_time

	-------------------------------------------
	-- extract timeout param
	-------------------------------------------
	if type(argv[argc]) == "number" then
		timeout = table.remove(argv)
	end
	
	-------------------------------------------
	-- add flags to the waiting-flag-list
	-------------------------------------------
	for k,v in ipairs(argv) do
	    if not add_shared_strlist(waiting_flaglist,v,true) then
			log_msg("add_shared_strlist() failed",file_id)
			return false
		end
	end

	-------------------------------------------
	-- add the taskid to the waiting-task-list
	-------------------------------------------
    if not add_shared_strlist(cmn_event_tasklist,g_taskid,true) then
		log_msg("add_shared_strlist() failed",file_id)
		return false
	end


	---------------------------------------------------
	-- waiting for clear all waiting-flag-list entries
	---------------------------------------------------
	while true do

		w_time = os.time()
		----------------------------------------------------------
		-- waiting for cmn_event_flag by event_set2()
		----------------------------------------------------------
		-- wstat = event_wait(cmn_event_flag,timeout)
		wstat = event_wait_common(timeout)
		if not wstat then break end

		----------------------------------------------------------
		-- check if all-waiting-flag were removed by event_set2()
		----------------------------------------------------------
		local stat,count = count_shared_strlist(waiting_flaglist)
		if count <= 0 then break end

		-- wait_time(1) -- insert a 1ms delay to avoid the bug of continuous calls of event_wait_common()

		--------------------------
		-- calc new timeout value
		--------------------------
		if timeout ~= -1 then
			local t = os.time()
			timeout = timeout - 1000 * os.difftime(t,w_time)
			if timeout <= 0 then
				wstat = false
				break
			end
		end

	end

	-----------------------------------
	-- remove taskid from waiting-task-list
	-----------------------------------
    if not remove_shared_strlist(cmn_event_tasklist,g_taskid) then
		log_msg("remove_shared_strlist() failed, ch = " .. cmn_event_tasklist .. " val = " .. g_taskid,file_id)
		return false
	end

	-----------------------------------
	-- clean-up the waiting-flag-list
	-----------------------------------
    if not clear_shared_strlist(waiting_flaglist) then
		log_msg("clear_shared_strlist() failed",file_id)
		return false
	end


	return wstat
end
]]

function event_wait_either2(...)
	return event_wait_either2x(...)
end

--[[
function event_wait_either2(...)
	local file_id = "event_wait_either2()"
	local timeout = -1
	local argv = {...}
	local argc = #argv
	local waiting_flaglist = cmn_event_flaglist_pre .. g_taskid
	local wstat
	local w_time
	local flag_count

	-------------------------------------------
	-- extract timeout param
	-------------------------------------------
	if type(argv[argc]) == "number" then
		timeout = table.remove(argv)
	end
	
	-------------------------------------------
	-- add flags to the waiting-flag-list
	-------------------------------------------
	flag_count = #argv
	for k,v in ipairs(argv) do
	    if not add_shared_strlist(waiting_flaglist,v,true) then
			log_msg("add_shared_strlist() failed",file_id)
			return false
		end
	end

	-------------------------------------------
	-- add the taskid to the waiting-task-list
	-------------------------------------------
    if not add_shared_strlist(cmn_event_tasklist,g_taskid,true) then
		log_msg("add_shared_strlist() failed",file_id)
		return false
	end


	---------------------------------------------------
	-- waiting for clear all waiting-flag-list entries
	---------------------------------------------------
	while true do

		w_time = os.time()
		----------------------------------------------------------
		-- waiting for cmn_event_flag by event_set2()
		----------------------------------------------------------
		-- wstat = event_wait(cmn_event_flag,timeout)
		wstat = event_wait_common(timeout)
		if not wstat then break end

		----------------------------------------------------------
		-- check if any waiting-flag was removed by event_set2()
		----------------------------------------------------------
		local stat,count = count_shared_strlist(waiting_flaglist)
		if count < flag_count then break end

		-- wait_time(1) -- insert a 1ms delay to avoid the bug of continuous calls of event_wait_common()

		--------------------------
		-- calc new timeout value
		--------------------------
		if timeout ~= -1 then
			local t = os.time()
			timeout = timeout - 1000 * os.difftime(t,w_time)
			if timeout <= 0 then
				wstat = false
				break
			end
		end

	end

	-----------------------------------
	-- remove taskid from waiting-task-list
	-----------------------------------
    if not remove_shared_strlist(cmn_event_tasklist,g_taskid) then
		log_msg("remove_shared_strlist() failed, ch = " .. cmn_event_tasklist .. " val = " .. g_taskid,file_id)
		return false
	end

	-----------------------------------
	-- clean-up the waiting-flag-list
	-----------------------------------
    if not clear_shared_strlist(waiting_flaglist) then
		log_msg("clear_shared_strlist() failed",file_id)
		return false
	end


	return wstat
end
]]

--[[
********************************************************************

	stat = event_set2(flag)

********************************************************************
]]

function event_set2(...)
	return event_set2x(...)
end

--[[
function event_set2(flag)
	local file_id = "event_set2()"
	local waiting_flaglist

	------------------------
	-- get waiting-task-list
	------------------------
	local stat,tasklist = get_shared_strlist(cmn_event_tasklist,false);
	if not stat then
		log_msg("get_shared_strlist() failed",file_id)
		return false
	end

	-------------------------------------------
	-- remove flags from the waiting-flag-list
	-------------------------------------------
	for k,v in ipairs(tasklist) do
		waiting_flaglist = cmn_event_flaglist_pre .. v
	    remove_shared_strlist(waiting_flaglist,flag) -- ignore no-channel error
	end

	-------------------------------------------
	-- set common event flag
	-------------------------------------------
	-- if not event_set(cmn_event_flag) then
	if not event_set_common() then
		log_msg("event_set() failed",file_id)
		return false
	end

	return true
end
]]

