
--[[
********************************************************************
	
	CRITICAL2_LIB.lua

	ver1.0 initial revision
	ver1.1 use "no_global_watch" flag

	copyright(c) 2014-2016 All Blue System 

********************************************************************
]]

--[[
********************************************************************

	stat = critical_section_enter2(key [,try_count])

	key:String		key name
	try_count 		maximum wait count
					if omitted, it will be wait indefinitely
	stat:boolean
			True		successfully entered into the critical section
			False		try_count has been exceeded or any other errors occurred

********************************************************************
]]

function critical_section_enter2(...)
	local try_count = 0
	local argv = {...}
	local argc = #argv
	local key
	local val = "0"
	local stat

	-------------------------------------------
	-- extract try_count param
	-------------------------------------------
	if type(argv[argc]) == "number" then
		try_count = table.remove(argv)
	end

	-------------------------------------------
	-- extract key param
	-------------------------------------------
	if type(argv[1]) == "string" then
		key = argv[1]
	else
		return false
	end

	---------------------------------------------------------------
	-- wait until the acquisition of the critical section of "key"
	---------------------------------------------------------------
	repeat
		wait_time(1)
		if try_count > 0 then
			if tonumber(val) > try_count then return false end
		end
		stat,val = inc_shared_data(key,true)
		if not stat then return false end
	until val == "1"

	return true
end

--[[
********************************************************************

	stat = critical_section_leave2(key)

	key:String		key name
	stat:boolean
			True		successfully leaved from the critical section
			False		error occurred

********************************************************************
]]

function critical_section_leave2(...)
	local argv = {...}
	local argc = #argv
	local key
	local stat

	-------------------------------------------
	-- extract key param
	-------------------------------------------
	if type(argv[1]) == "string" then
		key = argv[1]
	else
		return false
	end

	---------------------------------------------------------------
	-- release the critical section of "key"
	---------------------------------------------------------------
	stat = set_shared_data(key,"",true)

	return stat
end


