
--[[
********************************************************************
	
	EXCLUSIVE_LIB.lua

	ver1.1 ランフラグのキー名先頭に "$" を追加した

	ver1.0 initial revision

	copyright(c) 2014-2021 All Blue System 

********************************************************************
]]

--[[

	２重起動防止用チェック関数

	stat = exclusive_check(script_name)

	script_name:String		スクリプト名, コール側は常に g_script グローバル変数を指定すること

	stat:boolean

			true		この関数をコールしたタスク・スレッドは唯一このサーバー上で実行中

			false		このサーバー上で同一のスクリプトが別タスク・スレッドとして既に実行中

]]

function exclusive_check(script_name)
	local stat,tasklist,namelist,val
	local run_flag = "$" .. script_name .. "_RUNNING"
	stat, tasklist, namelist = script_task_list() -- 同じスクリプトが現在実行中かどうかを調べる
	local found_count = 0
	for k,v in ipairs(namelist) do
		if v == script_name then
			found_count = found_count + 1
		end
	end
	if found_count > 1 then -- 自身のタスクスレッドがあるため必ず 1 以上になる点に注意
		return false
	else
		-- 以前にこのスクリプトを強制終了した場合に残っている run_flag を消去する(もしあれば)
		stat,val = set_shared_data(run_flag,"")
	end

	-- スクリプト実行中かどうかを run_flag で再び調べる
	stat,val = get_shared_data(run_flag)
	if val ~= "" then
		return false
	end

	-- このタスクスレッドが１番乗りの場合以外は終了する
	stat,val = inc_shared_data(run_flag)
	if val ~= "1" then
		return false
	end

	return true

end


--[[

	２重起動防止用フラグクリア
	"agent_task -k <task_id>" コマンドで強制終了した場合には、通常この関数コール部分は実行されないので
	次回起動時に、exclusive_check() 関数中でこの関数と同等の機能が実行される

	exclusive_clean(script_name)

	script_name:String		スクリプト名, コール側は常に g_script グローバル変数を指定すること

]]

function exclusive_clean(script_name)
	local run_flag = "$" .. script_name .. "_RUNNING"
	set_shared_data(run_flag,"")
end


