
--[[

GRAPHIC2_PRINT.lua

change history:

ver1.10	2023/11/20  max_x,max_y リターン値追加
					X 軸方向の描画サイズ計算の誤りで終端未満で文字列を折り返していたのを修正

ver1.00 2022/2/21	初版

                                                           abs_agent copyright(c) 2022 All Blue System
]]


--[[
--------------------------------------------------------------------------------------------------------
指定したフォントで文字列を描画する。

stat, max_x, max_y = graphic2_print(channel, x, y, str, color [,font [,size [,sp_h [,page] ] ] ])

Change Log:

	2023/11/20  max_x,max_y リターン値追加
				X 軸方向の描画サイズ計算の誤りで終端未満で文字列を折り返していたのを修正

                                                        abs_agent   copyright(c) 2023   All Blue System
---------------------------------------------------------------------------------------------------------
]]

function graphic2_print(...)
	local argv = {...}
	local argc = #argv
	if (argc < 5) or (argc > 9) then return false end

	local font = "SHINONOME_FONTX"
	local font_w,font_h,e_font_w,e_font_h
	local font_ascii
	local use_e_font = false
	local size = 1 -- 拡大倍率
	local sp_h = 0 -- 縦方向の文字間隔
	local page = 0 -- 描画対象ページ
	local processed = false
	local max_x = -9999 -- 描画エリアの右端座標
	local max_y = -9999 -- 描画エリアの下端座標

	local ch = argv[1]
	local x0 = argv[2]
	local y0 = argv[3]
	local str = argv[4]
	local fg_color = argv[5]
	if argc >= 6 then font = argv[6] end
	if argc >= 7 then size = argv[7] end
	if argc >= 8 then sp_h = argv[8] end
	if argc >= 9 then page = argv[9] end

	------------------------------
	-- ディスプレイパラメータ取得
	------------------------------
	local disp_stat,scr_w,scr_h,scr_pages,pixel_depth = graphic2_init(ch)
	if not disp_stat then return false end

	------------------------------
	-- フォントパラメータチェック
	------------------------------
	fontx_try_init()
	local stat,font_info = shmem_copy(font,14,3) -- font_info[1]:width, font_info[2]:height, font_info[3]:Code 0-ASCII 1-SJIS
	if not stat then return false end
	font_w = font_info[1]
	font_h = font_info[2]
	font_ascii = (font_info[3] == 0)

	---------------------------------------------------------------------------------
	-- font が SJIS フォントテーブルの場合には、別途 ASCII フォントの存在をチェック
	---------------------------------------------------------------------------------
	local e_font = "E_" .. font
	if (not font_ascii) and find_shared_strlist("$FONTX_LIST",e_font) then
		local stat,e_font_info = shmem_copy(e_font,14,3)
		if stat and (e_font_info[3] == 0) then
			use_e_font = true
			e_font_w = e_font_info[1]
			e_font_h = e_font_info[2]
		end
	else
		e_font = ""
	end
 
	---------------------
	-- 表示開始位置計算
	---------------------
	if not x0 then x0 = 0 end
	if not y0 then y0 = 0 end
	if (x0 > scr_w) or (y0 > scr_h) then return true,max_x,max_y end
	if (page >= scr_pages) then return false end

	---------------------------------------------------------------------------------------------------------
	-- 文字列を全てバイトコード(ASCII コード)として解釈して、ASCII フォントのみを使用して表示
	---------------------------------------------------------------------------------------------------------
	if font_ascii then
		processed = true
		local ch_arr = str_to_tbl(str)
		if #ch_arr == 0 then return false end
		for k,v in ipairs(ch_arr) do
			local gstat, fdata = shmem_get_fontx(font,v)
			if not gstat then goto NEXT_CHAR1 end
			if x0 > scr_w - (font_w * size) then
				x0 = 0
				y0 = y0 + (sp_h + (font_h * size))
			end
			if y0 > (scr_h - (sp_h + (font_h * size))) then break end
			graphic2_draw_fontx(ch,x0,y0,font_w,font_h,tbl_to_hex(fdata),fg_color,size,page)
			x0 = x0 + (font_w * size)
			if x0 > max_x then max_x = x0 end
			if (y0 + font_h * size) > max_y then max_y = y0 + font_h * size end
::NEXT_CHAR1::
		end
	end

	---------------------------------------------------------------------------------------------------------
	-- 文字列を文字毎に分解した後、バイトコード(ASCII)文字については同じ字形のマルチバイト文字に変換した後、
	-- 全てSJIS フォントのみを使用して表示
	---------------------------------------------------------------------------------------------------------
	if (not font_ascii) and (not use_e_font) then
		processed = true
		local stat,ch_arr = str_to_tbl2(str,2,0x3f)
		if not stat then return false end
		if #ch_arr == 0 then return false end
		fontx_ascii_to_zenkaku(ch_arr)	-- ASCII コード部分を同じ字形の全角文字に変換
		for k,v in ipairs(ch_arr) do
			local gstat, fdata = shmem_get_fontx(font,v)
			if not gstat then goto NEXT_CHAR2 end
			if x0 > scr_w - (font_w * size) then
				x0 = 0
				y0 = y0 + (sp_h + (font_h * size))
			end
			if y0 > (scr_h - (sp_h + (font_h * size))) then break end
			graphic2_draw_fontx(ch,x0,y0,font_w,font_h,tbl_to_hex(fdata),fg_color,size,page)
			x0 = x0 + (font_w * size)
			if x0 > max_x then max_x = x0 end
			if (y0 + font_h * size) > max_y then max_y = y0 + font_h * size end
::NEXT_CHAR2::
		end
	end

	---------------------------------------------------------------------------------------------------------
	-- マルチバイト文字には SJIS フォントを使用して ASCII 文字には半角フォントを使用して表示
	---------------------------------------------------------------------------------------------------------
	if (not font_ascii) and use_e_font then
		local gstat,fdata
		processed = true
		local stat,ch_arr = str_to_tbl2(str,2,0x3f)
		if not stat then return false end
		if #ch_arr == 0 then return false end
		for k,v in ipairs(ch_arr) do
			if v < 0xff then
				gstat, fdata = shmem_get_fontx(e_font,v)
				if not gstat then goto NEXT_CHAR3 end
				if x0 > scr_w - (e_font_w * size) then
					x0 = 0
					y0 = y0 + (sp_h + (e_font_h * size))
				end
				if y0 > (scr_h - (sp_h + (e_font_h * size))) then break end
				graphic2_draw_fontx(ch,x0,y0,e_font_w,e_font_h,tbl_to_hex(fdata),fg_color,size,page)
				x0 = x0 + (e_font_w * size)
				if x0 > max_x then max_x = x0 end
				if (y0 + e_font_h * size) > max_y then max_y = y0 + e_font_h * size end
			else
				gstat, fdata = shmem_get_fontx(font,v)
				if not gstat then goto NEXT_CHAR3 end
				if x0 > scr_w - (font_w * size) then
					x0 = 0
					y0 = y0 + (sp_h + (font_h * size))
				end
				if y0 > (scr_h - (sp_h + (font_h * size))) then break end
				graphic2_draw_fontx(ch,x0,y0,font_w,font_h,tbl_to_hex(fdata),fg_color,size,page)
				x0 = x0 + (font_w * size)
				if x0 > max_x then max_x = x0 end
				if (y0 + font_h * size) > max_y then max_y = y0 + font_h * size end
			end
::NEXT_CHAR3::
		end
	end

	return processed,max_x,max_y
end
