
--[[
****************************************************************************************************

●機能概要

MGCP デバイスに接続する ST7565コントローラを搭載した Graphic LCDディスプレイ用ライブラリ

copyright(c) 2019 All Blue System

****************************************************************************************************
]]


--[[
****************************************************************************************************

デバイスを初期化する

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)
		rs_pin:number	LCD デバイスの RS(RegisterSelect) ピンに割り当てた GPIO ポート番号

リターン値
		無し

****************************************************************************************************
]]

function mgcp_AQM1248A_init(node,port,rs_pin)
	-- Graphic LCD module initialize
	if not graphic_set_size(128,48) then error() end
	if not graphic_clear() then error() end

	-- set RS to Low
	if not mgcp_gpio_set(node,rs_pin,0) then return false end

	-- AQM1248A initialize 
	if not mgcp_spi_write_block(node,port,2,-1,0,'"AEA0C8A32C","2E","2F","23811CA440A6AF"') then error() end
end


--[[
****************************************************************************************************

デバイスのブートカウンタを元に初期化が必要かどうかを判断する。
初期化が必要な場合のみ初期化動作を行う

パラメータ
		node:string		MGCP ノード名
		port:number		SPIポート番号 0 .. 1
		rs_pin:number	LCD デバイスの RS(RegisterSelect) ピンに割り当てた GPIO ポート番号

リターン値
		無し

****************************************************************************************************
]]

function mgcp_AQM1248A_try_init(node,port,rs_pin)
	local stat,reboot,check_key,flag
	check_key = "MGCP_AQM1248A_INIT_" .. node .. "_" .. tostring(port)
	stat,reboot = get_shared_data("$REBOOT_" .. node)
	if reboot == "" then
		mgcp_AQM1248A_init(node,port,rs_pin)
		set_shared_data(check_key,"")
	else
		stat,flag = get_shared_data(check_key)
		if (flag == "") or (flag ~= reboot) then 
			mgcp_AQM1248A_init(node,port,rs_pin)
			set_shared_data(check_key,reboot)
		end
	end
end

--[[
****************************************************************************************************

LCD にグラフィックデータを表示する

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		rs_pin:number	LCD デバイスの RS(RegisterSelect) ピンに割り当てた GPIO ポート番号

リターン値
		無し

****************************************************************************************************
]]

function mgcp_AQM1248A_display(node,port,rs_pin)
	local stat,cmd_data = graphic_dump(1)	-- ST7565用のコントロールコマンドとピクセルデータを出力
	if not stat then error() end
	local hexstr_arr = ""
	for key,val in ipairs(cmd_data) do
		if key > 1 then
			hexstr_arr = hexstr_arr .. ','
		end
		hexstr_arr =  hexstr_arr .. '"' .. val .. '"'
	end
	if not mgcp_spi_write_block(node,port,0,rs_pin,0,hexstr_arr) then error() end
end
