
--[[
****************************************************************************************************

●機能概要

MGCP デバイスに接続する ST7032iコントローラを使用した LCDディスプレイ(8x2)用ライブラリ

copyright(c) 2018 All Blue System

****************************************************************************************************
]]


--[[
****************************************************************************************************

デバイスを初期化する

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)

リターン値
		無し

****************************************************************************************************
]]

function mgcp_ST7032i_LCD8X2_init(node,port,addr)
	if not mgcp_i2c_write_block(node,port,addr,'"0038","0039","0004","0014","0070","005E","006C"',1) then error() end
	-- LCDモジュール側で必要な 300 ms ディレイ
	wait_time(300)
	if not mgcp_i2c_write_block(node,port,addr,'"0038","000C","0001"',1) then error() end
end

--[[
****************************************************************************************************

デバイスのブートカウンタを元に初期化が必要かどうかを判断する。
初期化が必要な場合のみ初期化動作を行う

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)

リターン値
		無し

****************************************************************************************************
]]

function mgcp_ST7032i_LCD8X2_try_init(node,port,addr)
	local stat,reboot,check_key,flag
	check_key = "MGCP_ST7032i_INIT_" .. node .. "_" .. tostring(port) .. "_" .. addr
	stat,reboot = get_shared_data("$REBOOT_" .. node)
	if reboot == "" then
		set_shared_data(check_key,"")
		mgcp_ST7032i_LCD8X2_init(node,port,addr)
	else
		stat,flag = get_shared_data(check_key)
		if (flag == "") or (flag ~= reboot) then 
			set_shared_data(check_key,reboot)
			mgcp_ST7032i_LCD8X2_init(node,port,addr)
		end
	end
end

--[[
****************************************************************************************************

LCD 表示内容をクリアする

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)

リターン値
		無し

****************************************************************************************************
]]

function mgcp_ST7032i_LCD8X2_clear(node,port,addr)
	if not mgcp_i2c_write(node,port,addr,"0001") then error() end
end

--[[
****************************************************************************************************

LCD に文字列を表示する

パラメータ
		node:string		MGCP ノード名
		port:number		I2C ポート番号 0 .. 1
		addr:string		I2Cスレーブアドレス(16進数文字列 2桁)
		str:string		LCD に表示する文字列
		line:number		表示開始ライン 0..1
						省略時は表示開始ラインは 0 に設定され、LCDカラム終端で自動的に次の行に折り返す。

リターン値
		無し

****************************************************************************************************
]]

function mgcp_ST7032i_LCD8X2_display(node,port,addr,str,line)
	local columns = 8			-- LCD文字カラム数
	local hexdata = ''
	if (line == nil) or (line == 0) then
		hexdata = hexdata .. '"0080"' -- LCD#0 行目に出力
	else
		hexdata = hexdata .. '"00C0"' -- LCD#1 行目に出力
	end
	-----------------------------
	-- 文字列をバイト配列値に変換
	-----------------------------
	local arr = str_to_tbl(str)
	if (#arr > (2 * columns)) then error() end -- 文字数チェック
	for key,val in ipairs(arr) do
		---------------------------
		-- DDRAM にデータ書き込み
		---------------------------
		hexdata = hexdata .. ',"40' .. string.format("%2.2X",val) .. '"'

		---------------------------
		-- 文字列の自動折り返し出力
		---------------------------
		if (key == columns) and (line == nil) and (#arr > columns) then
			hexdata = hexdata .. ',"00C0"'
		end
	end
	if not mgcp_i2c_write_block(node,port,addr,hexdata,1) then error() end
end

